<?php // $Id: mdl_soapserver.class.php,v 1.2 2008/02/13 13:59:32 schewe Exp $

/**
 * Base class for SOAP protocol-specific server layer.
 *
 * @package Web Services
 * @version $Id: mdl_soapserver.class.php,v 1.2 2008/02/13 13:59:32 schewe Exp $
 * @author Open Knowledge Technologies - http://www.oktech.ca/
 * @author Justin Filip <jfilip@oktech.ca>
 * @author Kay Schewe <schewe@oncampus.de>
 */

//    require_once('../config.php');
    require_once($CFG->dirroot . '/ws/server.class.php');
    require_once($CFG->dirroot . '/lib/soap/nusoap.php');    

    define('MOODLE_WS_NAMESPACE', $CFG->wwwroot . '/ws/wsdl');
	define('DEBUG', true);

    if (DEBUG) ini_set('soap.wsdl_cache_enabled', '0');


/**
 * The SOAP server class.
 * 
 * @package Web Services
 * @author Open Knowledge Technologies - http://www.oktech.ca/
 * @author Justin Filip <jfilip@oktech.ca>
 */
    class mdl_soapserver extends server {

    /**
     * Constructor method.
     * 
     * @param none
     * @return none
     */
        //function soapserver() {
        function __construct() {
        /// Necessary for processing any DB upgrades.
            parent::server();

            if (DEBUG) $this->debug_output('    Version: ' . $this->version);
            if (DEBUG) $this->debug_output('    Session Timeout: ' . $this->sessiontimeout);
        }

    /**
     * Initializes a new server and calls the dispatch function upon an
     * incoming client request.
     * 
     * @uses $CFG
     * @param mixed $httpdata HTTP_REQUEST data.
     * @return none
     */
        function main($httpdata) {
            global $CFG;

            if (DEBUG) $debug = 1;

        /// Start new SOAP server.
            if (strstr($_SERVER['QUERY_STRING'], 'wsdl') === false) {
                $sserver = new soap_server($CFG->wwwroot . '/ws/wsdl.php');

        /// Start a new SOAP server and register all the data types and service methods
        /// for dynamic WSDL creation.
            } else {
                $sserver = new soap_server();
                $sserver->configureWSDL('MoodleWS', MOODLE_WS_NAMESPACE, $CFG->wwwroot . '/ws/service.php?type=soap');
                $sserver->wsdl->schemaTargetNamespace = MOODLE_WS_NAMESPACE;


    /**  =====================  DEFINE NEW DATA TYPES  =======================  **/


            /// Define the structure for a user record.
                $sserver->wsdl->addComplexType(
                    'userRecord',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'error'        => array('name' => 'error', 'type' => 'xsd:string'),
                        'id'           => array('name' => 'id', 'type' => 'xsd:integer'),
                        'confirmed'    => array('name' => 'confirmed', 'type' => 'xsd:integer'),
                        'policyagreed' => array('name' => 'policyagreed', 'type' => 'xsd:integer'),
                        'deleted'      => array('name' => 'deleted', 'type' => 'xsd:integer'),
                        'username'     => array('name' => 'username', 'type' => 'xsd:string'),
                        'password'     => array('name' => 'password', 'type' => 'xsd:string'),
                        'idnumber'     => array('name' => 'idnumber', 'type' => 'xsd:string'),
                        'firstname'    => array('name' => 'firstname', 'type' => 'xsd:string'),
                        'lastname'     => array('name' => 'lastname', 'type' => 'xsd:string'),
                        'email'        => array('name' => 'email', 'type' => 'xsd:string'),
                        'icq'          => array('name' => 'icq', 'type' => 'xsd:string'),
                        'skype'        => array('name' => 'skype', 'type' => 'xsd:string'),
                        'yahoo'        => array('name' => 'yahoo', 'type' => 'xsd:string'),
                        'aim'          => array('name' => 'aim', 'type' => 'xsd:string'),
                        'msn'          => array('name' => 'msn', 'type' => 'xsd:string'),
                        'phone1'       => array('name' => 'phone1', 'type' => 'xsd:string'),
                        'phone2'       => array('name' => 'phone2', 'type' => 'xsd:string'),
                        'institution'  => array('name' => 'institution', 'type' => 'xsd:string'),
                        'department'   => array('name' => 'department', 'type' => 'xsd:string'),
                        'address'      => array('name' => 'address', 'type' => 'xsd:string'),
                        'city'         => array('name' => 'city', 'type' => 'xsd:string'),
                        'country'      => array('name' => 'country', 'type' => 'xsd:string'),
                        'lang'         => array('name' => 'lang', 'type' => 'xsd:string'),
                        'timezone'     => array('name' => 'timezone', 'type' => 'xsd:integer'),
                        'lastip'       => array('name' => 'lastip', 'type' => 'xsd:string'),
                        'description'  => array('name' => 'description', 'type' => 'xsd:string'),
						'theme'		   => array('name' => 'theme', 'type' => 'xsd:string'),
                        'emailstop'     => array('name' => 'emailstop', 'type' => 'xsd:integer'),
						'birthdate'		   => array('name' => 'birthdate', 'type' => 'xsd:string'),
						'company'		   => array('name' => 'company', 'type' => 'xsd:string')
                    )
                );

            /// Define the array of user records.
                $sserver->wsdl->addComplexType(
                    'userRecords',
                    'complexType', 
                    'array', 
                    '', 
                    'http://schemas.xmlsoap.org/soap/encoding/:Array', 
                    array(),
                    array(
                        array(
                            'ref' => 'SOAP-ENC:arrayType', 
                            'wsdl:arrayType' => 'tns:userRecord[]'
                        )
                    ),
                    'tns:userRecord'
                );

            /// Define the structure for user input data.
                $sserver->wsdl->addComplexType(
                    'userDatum',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'action'       => array('name' => 'action', 'type' => 'xsd:string'),
                        'confirmed'    => array('name' => 'confirmed', 'type' => 'xsd:integer'),
                        'policyagreed' => array('name' => 'policyagreed', 'type' => 'xsd:integer'),
                        'deleted'      => array('name' => 'deleted', 'type' => 'xsd:integer'),
                        'username'     => array('name' => 'username', 'type' => 'xsd:string'),
                        'password'     => array('name' => 'password', 'type' => 'xsd:string'),
                        'idnumber'     => array('name' => 'idnumber', 'type' => 'xsd:string'),
                        'firstname'    => array('name' => 'firstname', 'type' => 'xsd:string'),
                        'lastname'     => array('name' => 'lastname', 'type' => 'xsd:string'),
                        'email'        => array('name' => 'email', 'type' => 'xsd:string'),
                        'icq'          => array('name' => 'icq', 'type' => 'xsd:string'),
                        'skype'        => array('name' => 'skype', 'type' => 'xsd:string'),
                        'yahoo'        => array('name' => 'yahoo', 'type' => 'xsd:string'),
                        'aim'          => array('name' => 'aim', 'type' => 'xsd:string'),
                        'msn'          => array('name' => 'msn', 'type' => 'xsd:string'),
                        'phone1'       => array('name' => 'phone1', 'type' => 'xsd:string'),
                        'phone2'       => array('name' => 'phone2', 'type' => 'xsd:string'),
                        'institution'  => array('name' => 'institution', 'type' => 'xsd:string'),
                        'department'   => array('name' => 'department', 'type' => 'xsd:string'),
                        'address'      => array('name' => 'address', 'type' => 'xsd:string'),
                        'city'         => array('name' => 'city', 'type' => 'xsd:string'),
                        'country'      => array('name' => 'country', 'type' => 'xsd:string'),
                        'lang'         => array('name' => 'lang', 'type' => 'xsd:string'),
                        'timezone'     => array('name' => 'timezone', 'type' => 'xsd:integer'),
                        'lastip'       => array('name' => 'lastip', 'type' => 'xsd:string'),
                        'description'  => array('name' => 'description', 'type' => 'xsd:string'),
						'theme'		   => array('name' => 'theme', 'type' => 'xsd:string'),
                        'emailstop'     => array('name' => 'emailstop', 'type' => 'xsd:integer'),
                        'customerrole'  => array('name' => 'customerrole', 'type' => 'xsd:string'),
                        'birthdate'  => array('name' => 'birthdate', 'type' => 'xsd:string'),
                        'company'  => array('name' => 'company', 'type' => 'xsd:string')
                    )
                );

            /// Define the array of user input data.
                $sserver->wsdl->addComplexType(
                    'userData',
                    'complexType', 
                    'array', 
                    '', 
                    'http://schemas.xmlsoap.org/soap/encoding/:Array', 
                    array(),
                    array(
                        array(
                            'ref' => 'SOAP-ENC:arrayType', 
                            'wsdl:arrayType' => 'tns:userDatum[]'
                        )
                    ),
                    'tns:userDatum'
                );

            /// Define the structure for a course record.
                $sserver->wsdl->addComplexType(
                    'courseRecord',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'error'          => array('name' => 'error', 'type' => 'xsd:string'),
                        'id'             => array('name' => 'id', 'type' => 'xsd:integer'),
                        'category'       => array('name' => 'category', 'type' => 'xsd:integer'),
                        'categoryname'       => array('name' => 'categoryname', 'type' => 'xsd:string'),
                        'sortorder'      => array('name' => 'sortorder', 'type' => 'xsd:integer'),
                        'password'       => array('name' => 'password', 'type' => 'xsd:string'),
                        'fullname'       => array('name' => 'fullname', 'type' => 'xsd:string'),
                        'shortname'      => array('name' => 'shortname', 'type' => 'xsd:string'),
                        'idnumber'       => array('name' => 'shortname', 'type' => 'xsd:string'),
                        'summary'        => array('name' => 'summary', 'type' => 'xsd:string'),
                        'format'         => array('name' => 'format', 'type' => 'xsd:string'),
                        'showgrades'     => array('name' => 'showgrades', 'type' => 'xsd:integer'),
                        'newsitems'      => array('name' => 'newsitems', 'type' => 'xsd:integer'),
                        'teacher'        => array('name' => 'teacher', 'type' => 'xsd:string'),
                        'teachers'       => array('name' => 'teachers', 'type' => 'xsd:string'),
                        'student'        => array('name' => 'student', 'type' => 'xsd:string'),
                        'students'       => array('name' => 'students', 'type' => 'xsd:string'),
                        'guest'          => array('name' => 'guest', 'type' => 'xsd:integer'),
                        'startdate'      => array('name' => 'startdate', 'type' => 'xsd:integer'),
                        'enrolperiod'    => array('name' => 'enrolperiod', 'type' => 'xsd:integer'),
                        'numsections'    => array('name' => 'numsections', 'type' => 'xsd:integer'),
                        'marker'         => array('name' => 'marker', 'type' => 'xsd:integer'),
                        'maxbytes'       => array('name' => 'maxbytes', 'type' => 'xsd:integer'),
                        'visible'        => array('name' => 'visible', 'type' => 'xsd:integer'),
                        'hiddensections' => array('name' => 'hiddensections', 'type' => 'xsd:integer'),
                        'groupmode'      => array('name' => 'groupmode', 'type' => 'xsd:integer'),
                        'groupmodeforce' => array('name' => 'groupmodeforce', 'type' => 'xsd:integer'),
                        'lang'           => array('name' => 'lang', 'type' => 'xsd:string'),
                        'theme'          => array('name' => 'theme', 'type' => 'xsd:string'),
                        'cost'           => array('name' => 'cost', 'type' => 'xsd:string'),
                        'timecreated'    => array('name' => 'timecreated', 'type' => 'xsd:integer'),
                        'timemodified'   => array('name' => 'timemodified', 'type' => 'xsd:integer'),
                        'metacourse'     => array('name' => 'metacourse', 'type' => 'xsd:integer')
                    )
                );

            /// Define the array of user records.
                $sserver->wsdl->addComplexType(
                    'courseRecords',
                    'complexType', 
                    'array', 
                    '', 
                    'http://schemas.xmlsoap.org/soap/encoding/:Array', 
                    array(),
                    array(
                        array(
                            'ref' => 'SOAP-ENC:arrayType', 
                            'wsdl:arrayType' => 'tns:courseRecord[]'
                        )
                    ),
                    'tns:courseRecord'
                );


            /// Define the structure for course input data.
                $sserver->wsdl->addComplexType(
                    'courseDatum',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'action'         => array('name' => 'action', 'type' => 'xsd:string'),
                        'id'             => array('name' => 'id', 'type' => 'xsd:integer'),
                        'category'       => array('name' => 'category', 'type' => 'xsd:integer'),
                        'sortorder'      => array('name' => 'sortorder', 'type' => 'xsd:integer'),
                        'password'       => array('name' => 'password', 'type' => 'xsd:string'),
                        'fullname'       => array('name' => 'fullname', 'type' => 'xsd:string'),
                        'shortname'      => array('name' => 'shortname', 'type' => 'xsd:string'),
                        'idnumber'       => array('name' => 'shortname', 'type' => 'xsd:string'),
                        'summary'        => array('name' => 'summary', 'type' => 'xsd:string'),
                        'format'         => array('name' => 'format', 'type' => 'xsd:string'),
                        'showgrades'     => array('name' => 'showgrades', 'type' => 'xsd:integer'),
                        'newsitems'      => array('name' => 'newsitems', 'type' => 'xsd:integer'),
                        'teacher'        => array('name' => 'teacher', 'type' => 'xsd:string'),
                        'teachers'       => array('name' => 'teachers', 'type' => 'xsd:string'),
                        'student'        => array('name' => 'student', 'type' => 'xsd:string'),
                        'students'       => array('name' => 'students', 'type' => 'xsd:string'),
                        'guest'          => array('name' => 'guest', 'type' => 'xsd:integer'),
                        'startdate'      => array('name' => 'startdate', 'type' => 'xsd:integer'),
                        'enrolperiod'    => array('name' => 'enrolperiod', 'type' => 'xsd:integer'),
                        'numsections'    => array('name' => 'numsections', 'type' => 'xsd:integer'),
                        'marker'         => array('name' => 'marker', 'type' => 'xsd:integer'),
                        'maxbytes'       => array('name' => 'maxbytes', 'type' => 'xsd:integer'),
                        'visible'        => array('name' => 'visible', 'type' => 'xsd:integer'),
                        'hiddensections' => array('name' => 'hiddensections', 'type' => 'xsd:integer'),
                        'groupmode'      => array('name' => 'groupmode', 'type' => 'xsd:integer'),
                        'groupmodeforce' => array('name' => 'groupmodeforce', 'type' => 'xsd:integer'),
                        'lang'           => array('name' => 'lang', 'type' => 'xsd:string'),
                        'theme'          => array('name' => 'theme', 'type' => 'xsd:string'),
                        'cost'           => array('name' => 'cost', 'type' => 'xsd:string'),
                        'timecreated'    => array('name' => 'timecreated', 'type' => 'xsd:integer'),
                        'timemodified'   => array('name' => 'timemodified', 'type' => 'xsd:integer'),
                        'metacourse'     => array('name' => 'metacourse', 'type' => 'xsd:integer'),
                        'enrollable'     => array('name' => 'enrollable', 'type' => 'xsd:integer')
                    )
                );

            /// Define the array of course input data.
                $sserver->wsdl->addComplexType(
                    'courseData',
                    'complexType', 
                    'array', 
                    '', 
                    'http://schemas.xmlsoap.org/soap/encoding/:Array', 
                    array(),
                    array(
                        array(
                            'ref' => 'SOAP-ENC:arrayType', 
                            'wsdl:arrayType' => 'tns:courseDatum[]'
                        )
                    ),
                    'tns:courseDatum'
                );


            /// Define a grade record.
                $sserver->wsdl->addComplexType(
                    'gradeRecord',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'name'      => array('name' => 'name', 'type' => 'xsd:string'),
                        'maxgrade'  => array('name' => 'maxgrade', 'type' => 'xsd:integer'),
                        'grade'     => array('name' => 'grade', 'type' => 'xsd:string'),
                        'percent'   => array('name' => 'percent', 'type' => 'xsd:float'),
                        'weight'    => array('name' => 'weight', 'type' => 'xsd:float'),
                        'weighted'  => array('name' => 'weighted', 'type' => 'xsd:float'),
                        'sortOrder' => array('name' => 'sortOrder', 'type' => 'xsd:integer')
                    )
                );


            /// Define the array of grade records.
                $sserver->wsdl->addComplexType(
                    'gradeRecords',
                    'complexType', 
                    'array', 
                    '', 
                    'http://schemas.xmlsoap.org/soap/encoding/:Array', 
                    array(),
                    array(
                        array(
                            'ref' => 'SOAP-ENC:arrayType', 
                            'wsdl:arrayType' => 'tns:gradeRecord[]'
                        )
                    ),
                    'tns:gradeRecord'
                );


            /// Define a grade stats record.
                $sserver->wsdl->addComplexType(
                    'gradeStatsRecord',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'gradeItems'  => array('name' => 'gradeItems', 'type' => 'xsd:integer'),
                        'allgrades'   => array('name' => 'allgrades', 'type' => 'xsd:string'),                    
                        'points'      => array('name' => 'points', 'type' => 'xsd:integer'),
                        'totalpoints' => array('name' => 'totalpoints', 'type' => 'xsd:integer'),
                        'percent'     => array('name' => 'percent', 'type' => 'xsd:float'),
                        'weight'      => array('name' => 'weight', 'type' => 'xsd:float'),
                        'weighted'    => array('name' => 'weighted', 'type' => 'xsd:float')
                    )
                );


            /// Define a student grade record.
                $sserver->wsdl->addComplexType(
                    'studentGradeRecord',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'error'    => array('name' => 'error', 'type' => 'xsd:string'),
                        'courseid' => array('name' => 'courseid', 'type' => 'xsd:string'),
                        'stats'    => array('name' => 'stats', 'type' => 'tns:gradeStatsRecord'),
                        'grades'   => array('name' => 'grades', 'type' => 'tns:gradeRecords')
                    )
                );


            /// Define the array of student grade records.
                $sserver->wsdl->addComplexType(
                    'studentGradeRecords',
                    'complexType',
                    'array',
                    '',
                    'http://schemas.xmlsoap.org/soap/encoding/:Array',
                    array(),
                    array(
                        array(
                            'ref' => 'SOAP-ENC:arrayType', 
                            'wsdl:arrayType' => 'tns:studentGradeRecord[]'
                        )
                    ),
                    'tns:studentGradeRecord'
                );


            /// Define a student record.
                $sserver->wsdl->addComplexType(
                    'studentRecord',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'userid'     => array('name' => 'userid', 'type' => 'xsd:integer'),
                        'course'     => array('name' => 'course', 'type' => 'xsd:integer'),
                        'timestart'  => array('name' => 'timestart', 'type' => 'xsd:integer'),
                        'timeend'    => array('name' => 'timeend', 'type' => 'xsd:integer'),
                        'timeaccess' => array('name' => 'timeaccess', 'type' => 'xsd:integer'),
                        'enrol'      => array('name' => 'enrol', 'type' => 'xsd:string')
                    )
                );

            /// Define the array of student records.
                $sserver->wsdl->addComplexType(
                    'studentRecords',
                    'complexType',
                    'array',
                    '',
                    'http://schemas.xmlsoap.org/soap/encoding/:Array',
                    array(),
                    array(
                        array(
                            'ref' => 'SOAP-ENC:arrayType', 
                            'wsdl:arrayType' => 'tns:studentRecord[]'
                        )
                    ),
                    'tns:studentRecord'
                );


    /**  =====================  EXPOSE SERVICE METHODS  ======================  **/

            /// Define the return value for the getSyncgrade() method.
                $sserver->wsdl->addComplexType(
                    'getSyncgradeReturn',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'id'     => array('name' => 'id', 'type' => 'xsd:integer'),
                        'user' => array('name' => 'user', 'type' => 'xsd:string'),
                        'course' => array('name' => 'course', 'type' => 'xsd:string'),
                        'grade'     => array('name' => 'grade', 'type' => 'xsd:integer'),
                        'gradetime'     => array('name' => 'gradetime', 'type' => 'xsd:integer')
                    )
                );

				/// Register the get_syngrade() method.
                $sserver->register(
                    'mdl_soapserver.get_syncgrade',
                    array(
                        'client' => 'xsd:integer',
                        'sesskey' => 'xsd:string'
                    ),
                    array('return' => 'tns:getSyncgradeReturn'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#get_syncgrade',
                    'rpc',
                    'literal',
                    'MoodleWS get_syncgrade'
                );	
				/// Register the del_syngrade() method.
                $sserver->register(
                    'mdl_soapserver.del_syncgrade',
                    array(
                        'client' => 'xsd:integer',
                        'sesskey' => 'xsd:string',
                        'id' => 'xsd:integer',
                    ),
                    array('return' => 'xsd:string'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#del_syncgrade',
                    'rpc',
                    'literal',
                    'MoodleWS del_syncgrade'
                );
            /// Define the return value for the login() method.
                $sserver->wsdl->addComplexType(
                    'loginReturn',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'client'     => array('name' => 'client', 'type' => 'xsd:integer'),
                        'sessionkey' => array('name' => 'sessionkey', 'type' => 'xsd:string'),
                    )
                );

            /// Register the login() method.
                $sserver->register(
                    'mdl_soapserver.login',
                    array(
                        'username' => 'xsd:string',
                        'password' => 'xsd:string'
                    ),
                    array('return' => 'tns:loginReturn'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#login',
                    'rpc',
                    'literal',
                    'MoodleWS Client Login'
                );


            /// Register the logout() method.
                $sserver->register(
                    'mdl_soapserver.logout',
                    array(
                        'client' => 'xsd:integer',
                        'sesskey' => 'xsd:string'
                    ),
                    array('return' => 'xsd:boolean'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#logout',
                    'rpc',
                    'literal',
                    'MoodleWS: Client Logout'
                );


            /// Define the input value for the edit_user() method.
                $sserver->wsdl->addComplexType(
                    'editUserInput',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'users' => array('name' => 'users', 'type' => 'tns:userData')
                    )
                );

            /// Define the return value for the edit_user() method.
                $sserver->wsdl->addComplexType(
                    'editUserOutput',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'users' => array('name' => 'users', 'type' => 'tns:userRecords')
                    )
                );

            /// Register the edit_user() method.
                $sserver->register(
                    'mdl_soapserver.edit_user',
                    array(
                        'client' => 'xsd:integer',
                        'sesskey' => 'xsd:string',
                        'users' => 'tns:editUserInput'
                    ),
                    array('return' => 'tns:editUserOutput'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#edit_user',
                    'rpc',
                    'literal',
                    'MoodleWS: Edit User Information'
                );


            /// Define the input value for the get_user() method.
                $sserver->wsdl->addComplexType(
                    'getUserInput',
                    'complexType', 
                    'struct', 
                    'sequence', 
                    '', 
                    array(
                        'userid' => array(
                            'name' => 'userid',
                            'type' => 'xsd:string',
                            'minOccurs' => '0',
                            'maxOccurs' => 'unbounded'
                        )
                    ),
                    array(),
                    'tns:getUserInput'
                );

            /// Define the return value for the get_user() method.
                $sserver->wsdl->addComplexType(
                    'getUserReturn',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'users' => array('name' => 'users', 'type' => 'tns:userRecords')
                    )
                );

            /// Register the get_user() method.
                $sserver->register(
                    'mdl_soapserver.get_user',
                    array(
                        'client' => 'xsd:integer',
                        'sesskey' => 'xsd:string',
                        'userids' => 'tns:getUserInput',
                        'idfield' => 'xsd:string'
                    ),
                    array('return' => 'tns:getUserReturn'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#get_user',
                    'rpc',
                    'literal',
                    'MoodleWS: Get User Information'
                );


            /// Define the input value for the edit_course() method.
                $sserver->wsdl->addComplexType(
                    'editCourseInput',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'courses' => array('name' => 'courses', 'type' => 'tns:courseData')
                    )
                );

            /// Define the return value for the edit_course() method.
                $sserver->wsdl->addComplexType(
                    'editCourseOutput',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'courses' => array('name' => 'courses', 'type' => 'tns:courseRecords')
                    )
                );

            /// Register the edit_course() method.
                $sserver->register(
                    'mdl_soapserver.edit_course',
                    array(
                        'client' => 'xsd:integer',
                        'sesskey' => 'xsd:string',
                        'courses' => 'tns:editCourseInput'
                    ),
                    array('return' => 'tns:editCourseOutput'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#edit_course',
                    'rpc',
                    'literal',
                    'MoodleWS: Edit Course Information'
                );


            /// Define the input value for the get_course() method.
                $sserver->wsdl->addComplexType(
                    'getCourseInput',
                    'complexType', 
                    'struct', 
                    'sequence', 
                    '', 
                    array(
                        'courseid' => array(
                            'name' => 'courseid',
                            'type' => 'xsd:string',
                            'minOccurs' => '0',
                            'maxOccurs' => 'unbounded'
                        )
                    ),
                    array(),
                    'tns:getCourseInput'
                );


            /// Define the return value for the get_course() method.
                $sserver->wsdl->addComplexType(
                    'getCourseReturn',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'courses' => array('name' => 'courses', 'type' => 'tns:courseRecords')
                    )
                );

            /// Register the get_course() method.
                $sserver->register(
                    'mdl_soapserver.get_course',
                    array(
                        'client' => 'xsd:integer',
                        'sesskey' => 'xsd:string',
                        'courseids' => 'tns:getCourseInput',
                        'idfield' => 'xsd:string'
                    ),
                    array('return' => 'tns:getCourseReturn'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#get_course',
                    'rpc',
                    'literal',
                    'MoodleWS: Get Course Information'
                );


            /// Define the input value for the get_grades() method.
                $sserver->wsdl->addComplexType(
                    'getGradesInput',
                    'complexType', 
                    'struct', 
                    'sequence', 
                    '', 
                    array(
                        'courseid' => array(
                            'name' => 'courseid',
                            'type' => 'xsd:string',
                            'minOccurs' => '0',
                            'maxOccurs' => 'unbounded'
                        )
                    ),
                    array(),
                    'tns:getCourseInput'
                );

            /// Define the return value for the get_grades() method.
                $sserver->wsdl->addComplexType(
                    'getGradesReturn',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'grades' => array('name' => 'studentGrades', 'type' => 'tns:studentGradeRecords')
                    )
                );

            /// Register the get_grades() method.
                $sserver->register(
                    'mdl_soapserver.get_grades',
                    array(
                        'client'    => 'xsd:integer',
                        'sesskey'   => 'xsd:string',
                        'userid'    => 'xsd:string',
                        'courseids' => 'tns:getGradesInput',
                        'idfield' => 'xsd:string'
                    ),
                    array('return' => 'tns:getGradesReturn'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#get_grades',
                    'rpc',
                    'literal',
                    'MoodleWS: Get User Grades'
                );


            /// Define the input value for the enrol_students() method.
                $sserver->wsdl->addComplexType(
                    'enrolStudentsInput',
                    'complexType', 
                    'struct', 
                    'sequence', 
                    '', 
                    array(
                        'userid' => array(
                            'name' => 'userid',
                            'type' => 'xsd:string',
                            'minOccurs' => '0',
                            'maxOccurs' => 'unbounded'
                        )
                    ),
                    array(),
                    'tns:enrolStudentsInput'
                );
    /// Define the input value for the enrol() method.
                $sserver->wsdl->addComplexType(
                    'enrolInput',
                    'complexType', 
                    'array', 
                    '', 
                    'SOAP-ENC:Array', 
                    array(),
                    array(
                        array(
                            'ref' => 'SOAP-ENC:arrayType', 
                            'wsdl:arrayType' => 'xsd:string'
                        )
                    ),
                    'tns:enrolInput'
                );
            /// Define the input value for the unenrol() method.
                $sserver->wsdl->addComplexType(
                    'unenrolInput',
                    'complexType', 
                    'array', 
                    '', 
                    'SOAP-ENC:Array', 
                    array(),
                    array(
                        array(
                            'ref' => 'SOAP-ENC:arrayType', 
                            'wsdl:arrayType' => 'xsd:string'
                        )
                    ),
                    'tns:unenrolInput'
                );
                

            /// Define the return value for the enrol_students() method.
                $sserver->wsdl->addComplexType(
                    'enrolStudentsReturn',
                    'complexType',
                    'struct',
                    'all',
                    '',
                    array(
                        'error'    => array('name' => 'error', 'type' => 'xsd:string'),
                        'students' => array('name' => 'studentRecords', 'type' => 'tns:studentRecords')
                    )
                );

            /// Register the enrol_students() method.
                $sserver->register(
                    'mdl_soapserver.enrol_students',
                    array(
                        'client'   => 'xsd:integer',
                        'sesskey'  => 'xsd:string',
                        'courseid' => 'xsd:string',
                        'userids'  => 'tns:enrolStudentsInput',
                        'idfield'  => 'xsd:string'
                    ),
                    array('return' => 'tns:enrolStudentsReturn'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#enrol_students',
                    'rpc',
                    'literal',
                    'MoodleWS: Enrol students in a course'
                );

          /// Register the enrol() method.
                $ok = $sserver->register(
                    'mdl_soapserver.enrol',
                    array(
                        'client'   => 'xsd:integer',
                        'sesskey'  => 'xsd:string',
                        'courseid' => 'xsd:string',
                        'userids'  => 'tns:enrolInput',
                        'roleid'  => 'xsd:string'
                    ),
                    array('return' => 'xsd:string'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#enrol',
                    'rpc',
                    'encoded',
                    'MoodleWS: Enrol users with role in a course'
                );

          /// Register the unenrol() method.
                $ok = $sserver->register(
                    'mdl_soapserver.unenrol',
                    array(
                        'client'   => 'xsd:integer',
                        'sesskey'  => 'xsd:string',
                        'courseid' => 'xsd:string',
                        'userids'  => 'tns:unenrolInput'
                    ),
                    array('return' => 'xsd:string'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#unenrol',
                    'rpc',
                    'encoded',
                    'MoodleWS: Unenrol users from a course'
                );
            
            /// Register the import_backup() method.
                $ok = $sserver->register(
                    'mdl_soapserver.import_backup',
                    array(
                        'client'   => 'xsd:integer',
                        'sesskey'  => 'xsd:string',
                        'courseid' => 'xsd:string',
                        'file'  => 'xsd:string'
                    ),
                    array('return' => 'xsd:string'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#import_backup',
                    'rpc',
                    'encoded',
                    'MoodleWS: import course backup'
                );
                   
            /// Register the add_to_metacourse() method.
                $ok = $sserver->register(
                    'mdl_soapserver.add_to_metacourse',
                    array(
                        'client'   => 'xsd:integer',
                        'sesskey'  => 'xsd:string',
                        'courseid1' => 'xsd:string',
                        'courseid2'  => 'xsd:string'
                    ),
                    array('return' => 'xsd:string'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#add_to_metacourse',
                    'rpc',
                    'encoded',
                    'MoodleWS: add course / meta course connection'
                );
           
           /// Register the add_resource_link() method.
                $ok = $sserver->register(
                    'mdl_soapserver.add_resource_link',
                    array(
                        'client'   => 'xsd:integer',
                        'sesskey'  => 'xsd:string',
                        'courseid' => 'xsd:string',
                        'url'  => 'xsd:string',
                        'name'  => 'xsd:string',
                        'summary'  => 'xsd:string',
                        'type'  => 'xsd:string'
                    ),
                    array('return' => 'xsd:string'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#add_resource_link',
                    'rpc',
                    'encoded',
                    'MoodleWS: add resource link to course'
                );
                
             /// Register the update_resource_repository() method.
                $ok = $sserver->register(
                    'mdl_soapserver.update_resource_link',
                    array(
                        'client'   => 'xsd:integer',
                        'sesskey'  => 'xsd:string',
                        'courseid' => 'xsd:string',
                        'url'  => 'xsd:string',
                        'name'  => 'xsd:string',
                        'summary'  => 'xsd:string',
                        'type'  => 'xsd:string',
                        'rname'  => 'xsd:string'
                    ),
                    array('return' => 'xsd:string'),
                    MOODLE_WS_NAMESPACE,
                    MOODLE_WS_NAMESPACE . '#update_resource_link',
                    'rpc',
                    'encoded',
                    'MoodleWS: update resource link in course'
                );
                
                if (!$ok) $this->debug_output('Could not register method');

                if (DEBUG) $this->debug_output('Initialized server methods.');
            }

        /// Use the request to (try to) invoke the service.
            $sserver->service($httpdata);
        }


/**  =====  OVERRIDE SERVER METHODS NEEDING SOAP-SPECIFIC HANDLING  ======  **/


        function get_syncgrade($client, $sesskey) {
		
		
		if ($syncgraderecord = parent::get_syncgrade($client, $sesskey)) {
			if ($syncgraderecord->id > 0) {
				$rgrade=array(
					'id'=>$syncgraderecord->id,
					'user'=>$syncgraderecord->user,
					'course'=>$syncgraderecord->course,
					'grade'=>$syncgraderecord->grade,
					'gradetime'=>$syncgraderecord->gradetime,
					);
				}
			else{
				$rgrade=array(
					'id'=>0,
					'user'=>'',
					'course'=>'',
					'grade'=>0,
					'gradetime'=>0,
					);
				}
			}
		
        return $rgrade;
        }

        function del_syncgrade($client, $sesskey, $id) {
		 return parent::del_syncgrade($client, $sesskey, $id);	  
        }



    /**
     * Edit user records (add/update/delete).
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param array $users An array of user records (objects or arrays) for editing
     *                     (including operation to perform).
     * @return array An array of user objects.
     */
        function edit_user($client, $sesskey, $users) {
            $return = array();

            if ($rusers = parent::edit_user($client, $sesskey, $users)) {
                foreach ($rusers as $user) {
                    if (empty($user->error)) {
                        $ruser['error']        = '';
                        $ruser['id']           = $user->id;
                        $ruser['confirmed']    = $user->confirmed;
                        $ruser['policyagreed'] = $user->policyagreed;
                        $ruser['deleted']      = $user->deleted;
                        $ruser['username']     = $user->username;
                        $ruser['password']     = $user->password;
                        $ruser['idnumber']     = $user->idnumber;
                        $ruser['firstname']    = $user->firstname;
                        $ruser['lastname']     = $user->lastname;
                        $ruser['email']        = $user->email;
                        $ruser['icq']          = $user->icq;
                        $ruser['skype']        = $user->skype;
                        $ruser['yahoo']        = $user->yahoo;
                        $ruser['aim']          = $user->aim;
                        $ruser['msn']          = $user->msn;
                        $ruser['phone1']       = $user->phone1;
                        $ruser['phone2']       = $user->phone2;
                        $ruser['institution']  = $user->institution;
                        $ruser['department']   = $user->department;
                        $ruser['address']      = $user->address;
                        $ruser['city']         = $user->city;
                        $ruser['country']      = $user->country;
                        $ruser['lang']         = $user->lang;
                        $ruser['timezone']     = $user->timezone;
                        //$ruser['lastip']       = $user->lastIP;
                        $ruser['description']  = $user->description;
                        $ruser['theme']  	   = $user->theme;
                        $ruser['emailstop']     = $user->emailstop;

                    } else {
                        $ruser['error']        = $user->error;
                        $ruser['id']           = 0;
                        $ruser['confirmed']    = '';
                        $ruser['policyagreed'] = 'false';
                        $ruser['deleted']      = 'false';
                        $ruser['username']     = '';
                        $ruser['password']     = '';
                        $ruser['idnumber']     = '';
                        $ruser['firstname']    = '';
                        $ruser['lastname']     = '';
                        $ruser['email']        = '';
                        $ruser['icq']          = '';
                        $ruser['skype']        = '';
                        $ruser['yahoo']        = '';
                        $ruser['aim']          = '';
                        $ruser['msn']          = '';
                        $ruser['phone1']       = '';
                        $ruser['phone2']       = '';
                        $ruser['institution']  = '';
                        $ruser['department']   = '';
                        $ruser['address']      = '';
                        $ruser['city']         = '';
                        $ruser['country']      = '';
                        $ruser['lang']         = '';
                        $ruser['timezone']     = 0;
                        //$ruser['lastip']       = '';
                        $ruser['description']  = '';
						$ruser['theme']  = '';
                        $ruser['emailstop']     = 0;
                    }

                    $return['users'][] = $ruser;
                }
            }

            return $return;
        }


    /**
     * Find and return a list of user records.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param array $userids An array of input user idnumber values.
     * @return array An array of user records.
     */
        function get_user($client, $sesskey, $userids, $idfield = 'idnumber') {
            $rusers = array();

            //if ($users = parent::get_user($client, $sesskey, $userids['userid'], $idfield)) {
		if ($users = parent::get_user($client, $sesskey, $userids, $idfield)) {
                foreach ($users as $user) {
                    if (empty($user->error)) {
                        $ruser = array(
                            'error'        => '',
                            'confirmed'    => $user->confirmed,
                            'policyagreed' => $user->policyagreed,
                            'deleted'      => $user->deleted,
                            'username'     => $user->username,
                            'password'     => $user->password,
                            'idnumber'     => $user->idnumber,
                            'firstname'    => $user->firstname,
                            'lastname'     => $user->lastname,
                            'email'        => $user->email,
                            'icq'          => $user->icq,
                            'skype'        => $user->skype,
                            'yahoo'        => $user->yahoo,
                            'aim'          => $user->aim,
                            'msn'          => $user->msn,
                            'phone1'       => $user->phone1,
                            'phone2'       => $user->phone2,
                            'institution'  => $user->institution,
                            'department'   => $user->department,
                            'address'      => $user->address,
                            'city'         => $user->city,
                            'country'      => $user->country,
                            'lang'         => $user->lang,
                            'timezone'     => $user->timezone,
                            //'lastip'       => $user->lastIP,
                            'description'  => $user->description,
                            'theme'  => $user->theme,
                            'emailstop'     => $user->emailstop
                        );

                    } else {
                        $ruser = array(
                            'error'        => $user->error,
                            'confirmed'    => '',
                            'policyagreed' => 'false',
                            'deleted'      => 'false',
                            'username'     => '',
                            'password'     => '',
                            'idnumber'     => '',
                            'firstname'    => '',
                            'lastname'     => '',
                            'email'        => '',
                            'icq'          => '',
                            'skype'        => '',
                            'yahoo'        => '',
                            'aim'          => '',
                            'msn'          => '',
                            'phone1'       => '',
                            'phone2'       => '',
                            'institution'  => '',
                            'department'   => '',
                            'address'      => '',
                            'city'         => '',
                            'country'      => '',
                            'lang'         => '',
                            'timezone'     => 0,
                            //'lastip'       => '',
                            'description'  => '',
							'theme'  => '',
                            'emailstop'     => 0
                        );
                    }

                    $rusers['users'][] = $ruser;
                }
            }

            return $rusers;
        }


    /**
     * Find Edit course records (add/update/delete).
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param array $courseids An array of course records (objects or arrays) for
     *                         editing (including opertaion to perform).
     * @return array An array of course records.
     */
        function edit_course($client, $sesskey, $courses) {
            $return = array();

            if ($rcourses = parent::edit_course($client, $sesskey, $courses)) {
                foreach ($rcourses as $course) {
                    if (empty($course->error)) {
                        $rcourse = array(
                            'error'          => '',
                            'id'             => $course->id,
                            'category'       => $course->category,
                            'sortorder'      => $course->sortorder,
                            'password'       => $course->password,
                            'fullname'       => $course->fullname,
                            'shortname'      => $course->shortname,
                            'idnumber'       => $course->idnumber,
                            'summary'        => $course->summary,
                            'format'         => $course->format,
                            'showgrades'     => $course->showgrades,
                            'newsitems'      => $course->newsitems,
                            'teacher'        => $course->teacher,
                            'teachers'       => $course->teachers,
                            'student'        => $course->student,
                            'students'       => $course->students,
                            'guest'          => $course->guest,
                            'startdate'      => $course->startdate,
                            'enrolperiod'    => $course->enrolperiod,
                            'numsections'    => $course->numsections,
                            'marker'         => $course->marker,
                            'maxbytes'       => $course->maxbytes,
                            'visible'        => $course->visible,
                            'hiddensections' => $course->hiddensections,
                            'groupmode'      => $course->groupmode,
                            'groupmodeforce' => $course->groupmodeforce,
                            'lang'           => $course->lang,
                            'theme'          => $course->theme,
                            'cost'           => $course->cost,
                            'timecreated'    => $course->timecreated,
                            'timemodified'   => $course->timemodified,
                            'metacourse'     => $course->metacourse
                        );

                    } else {
                        $rcourse = array(
                            'error'          => $course->error,
                            'id'             => 0,
                            'category'       => 0,
                            'sortorder'      => 0,
                            'password'       => '',
                            'fullname'       => '',
                            'shortname'      => '',
                            'idnumber'       => '',
                            'summary'        => '',
                            'format'         => '',
                            'showgrades'     => 0,
                            'newsitems'      => 0,
                            'teacher'        => '',
                            'teachers'       => '',
                            'student'        => '',
                            'students'       => '',
                            'guest'          => 0,
                            'startdate'      => 0,
                            'enrolperiod'    => 0,
                            'numsections'    => 0,
                            'marker'         => 0,
                            'maxbytes'       => 0,
                            'visible'        => 0,
                            'hiddensections' => 0,
                            'groupmode'      => 0,
                            'groupmodeforce' => 0,
                            'lang'           => '',
                            'theme'          => '',
                            'cost'           => '',
                            'timecreated'    => 0,
                            'timemodified'   => 0,
                            'metacourse'     => 0
                        );
                    }

                    $return['courses'][] = $rcourse;
                }
            }

            return $return;
        }


    /**
     * Find and return a list of course records.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param array $courseids An array of input course idnumber values.
     * @return array An array of course records.
     */
        function get_course($client, $sesskey, $courseids, $idfield) {
            $rcourses = array();

            if ($courses = parent::get_course($client, $sesskey, $courseids['courseid'], $idfield)) {
                foreach ($courses as $course) {
                    if (empty($course->error)) {
                        $rcourse = array(
                            'error'          => '',
                            'id'             => $course->id,
                            'category'       => $course->category,
                            'sortorder'      => $course->sortorder,
                            'password'       => $course->password,
                            'fullname'       => $course->fullname,
                            'shortname'      => $course->shortname,
                            'idnumber'       => $course->idnumber,
                            'summary'        => $course->summary,
                            'format'         => $course->format,
                            'showgrades'     => $course->showgrades,
                            'newsitems'      => $course->newsitems,
                            'teacher'        => $course->teacher,
                            'teachers'       => $course->teachers,
                            'student'        => $course->student,
                            'students'       => $course->students,
                            'guest'          => $course->guest,
                            'startdate'      => $course->startdate,
                            'enrolperiod'    => $course->enrolperiod,
                            'numsections'    => $course->numsections,
                            'marker'         => $course->marker,
                            'maxbytes'       => $course->maxbytes,
                            'visible'        => $course->visible,
                            'hiddensections' => $course->hiddensections,
                            'groupmode'      => $course->groupmode,
                            'groupmodeforce' => $course->groupmodeforce,
                            'lang'           => $course->lang,
                            'theme'          => $course->theme,
                            'cost'           => $course->cost,
                            'timecreated'    => $course->timecreated,
                            'timemodified'   => $course->timemodified,
                            'metacourse'     => $course->metacourse
                        );

                    } else {
                        $rcourse = array(
                            'error'          => $course->error,
                            'id'             => 0,
                            'category'       => 0,
                            'sortorder'      => 0,
                            'password'       => '',
                            'fullname'       => '',
                            'shortname'      => '',
                            'idnumber'       => '',
                            'summary'        => '',
                            'format'         => '',
                            'showgrades'     => 0,
                            'newsitems'      => 0,
                            'teacher'        => '',
                            'teachers'       => '',
                            'student'        => '',
                            'students'       => '',
                            'guest'          => 0,
                            'startdate'      => 0,
                            'enrolperiod'    => 0,
                            'numsections'    => 0,
                            'marker'         => 0,
                            'maxbytes'       => 0,
                            'visible'        => 0,
                            'hiddensections' => 0,
                            'groupmode'      => 0,
                            'groupmodeforce' => 0,
                            'lang'           => '',
                            'theme'          => '',
                            'cost'           => '',
                            'timecreated'    => 0,
                            'timemodified'   => 0,
                            'metacourse'     => 0
                        );
                    }

                    $rcourses['courses'][] = $rcourse;
                }
            }

            return $rcourses;
        }


    /**
     * Find and return a list of student grade values.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param string $userid The user ID number of the student.
     * @param array $courseids An array of input course idnumber values.
     * @return array An array of grade records.
     */
        function get_grades($client, $sesskey, $userid, $courseids, $idfield) {
            $rgrades = array();

            if ($grades = parent::get_grades($client, $sesskey, $userid, $courseids, $idfield)) {
                foreach ($grades as $grade) {
                    if (!empty($grade->error)) {
                        $rgrade             = array();
                        $rgrade['error']    = $grade->error;
                        $rgrade['courseid'] = $grade->courseid;
                        $rgrade['stats']    = array();
                        $rgrade['grades']   = array();

                    } else {
                        $rgrade             = array();
                        $rgrade['error']    = '';
                        //$rgrade['courseid'] = $grade->courseid;
                        $stats              = array();
                        $mygrades           = array();

                        if (!empty($grade->grades)) {
                            while ($mygrade = current($grade->grades)) {
                                if (key($grade->grades) == 'stats') {
                                    $stats = array(
                                        'gradeItems'  => (int)$mygrade['grade_items'],                                        
                                        'allgrades'   => $mygrade['allgrades'],
                                        'points'      => (int)$mygrade['points'],
                                        'totalpoints' => (int)$mygrade['totalpoints'],
                                        'percent'     => (float)$mygrade['percent'],
                                        'weight'      => (float)$mygrade['weight'],
                                        'weighted'    => (float)$mygrade['weighted'],
                                    );

                                } else {
                                    $graded = array(
                                        'name'      => key($grade->grades),
                                        'maxgrade'  => (int)$mygrade['maxgrade'],
                                        'grade'     => $mygrade['grade'],
                                        'percent'   => (float)$mygrade['percent'],
                                        'weight'    => (float)$mygrade['weight'],
                                        'weighted'  => (float)$mygrade['weighted'],
                                        'sortOrder' => (int)$mygrade['sort_order']
                                    );

                                    $mygrades[] = $graded;
                                }

                                next($grade->grades);
                            }
                        }

                        $rgrade['stats']  = $stats;
                        $rgrade['grades'] = $mygrades;
                    }

                    $rgrades['grades'][] = $rgrade;
                }
            }

            return $rgrades;
        }


    /**
     * Enrol users as a student in the given course.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param string $courseid The course ID number to enrol students in.
     * @param array $userids An array of input user idnumber values for enrolment.
     * @return array Return data (user_student records) to be converted into a
     *               specific data format for sending to the client.
     */
        function enrol_students($client, $sesskey, $courseid, $userids, $idfield) {
            $rstudents = array();

            if ($students = parent::enrol_students($client, $sesskey, $courseid, $userids, $idfield)) {
                if (!empty($students->error)) {
                    $rstudents['error']    = $students->error;
                    $rstudents['students'] = array();

                } else {
                    $rstudents['error']    = '';
                    $rstudents['students'] = 'success';//$students->students;
                }
            }

            return $rstudents;
        }


    /**
     * Enrol users with role in the given course.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param string $courseid The course ID number to enrol students in.
     * @param array $userids An array of input user idnumber values for enrolment.
     * @param string $roleid The role (student, teacher, editingteacher)
     * @return array Return data (user_student records) to be converted into a
     *               specific data format for sending to the client.
     */
        function enrol($client, $sesskey, $courseid, $userids, $roleid ) {
	    //parent::debug_output(__METHOD__.' '.var_export($userids, true));
            //$rusers = array();
            //error_log(__METHOD__, 1, "schewe@oncampus.de");
            $return = '';
            if ($users = parent::enrol($client, $sesskey, $courseid, $userids, $roleid)) {
                if (!empty($users->error)) {
                    //$rusers['error']    = $users->error;
                    //$rusers['users'] = array();
                    $return = 'error';
                } else {
                    //$rusers['error']    = '';
                    //$rusers['users'] = $users->users;
                    $return = 'success';
                }
            }
        
            return $return;//$rusers;
        }
     
//--------------------------------

     /**
     * Unenrol users from the given course.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param string $courseid The course ID number to enrol students in.
     * @param array $userids An array of input user idnumber values for enrolment.
     * @return array Return data (user_student records) to be converted into a
     *               specific data format for sending to the client.
     */
        function unenrol($client, $sesskey, $courseid, $userids) {
            //parent::debug_output(__METHOD__.' '.var_export($userids, true));
            //$rusers = array();
            //error_log(__METHOD__, 1, "schewe@oncampus.de");
            $return = '';
            if ($users = parent::unenrol($client, $sesskey, $courseid, $userids)) {
                if (!empty($users->error)) {
                    $return = 'error';
                } else {
                    $return = 'success';
                }
            }
        
            return $return;
        }
          
//--------------------------------
        function import_backup($client, $sesskey, $courseid, $file) {
            //error_log(__METHOD__, 1, "schewe@oncampus.de");
            $return = '';
            if ($result = parent::import_backup($client, $sesskey, $courseid, $file)) {
                if (!empty($result->error)) {
                    $return = 'error';
                } 
                else 
                {
                    $return = 'success';
                }
            }
        
            return $return;
        }
        
//--------------------------------

        function add_to_metacourse($client, $sesskey, $courseid1, $courseid2) {
            //error_log(__METHOD__, 1, "schewe@oncampus.de");
            $return = '';
            if ($result = parent::add_to_metacourse($client, $sesskey, $courseid1, $courseid2)) {
                if (!empty($result->error)) {
                    $return = 'error';
                } 
                else 
                {
                    $return = 'success';
                }
            }
        
            return $return;
        }
        
//--------------------------------

        function add_resource_link($client, $sesskey, $courseid, $url, $name, $summary = '', $type = 'file') {
            //error_log(__METHOD__, 1, "schewe@oncampus.de");
            $return = '';
            if ($result = parent::add_resource_link($client, $sesskey, $courseid, $url, $name, $summary, $type)) {
                if (!empty($result->error)) {
                    $return = 'error';
                } 
                else 
                {
                    $return = 'success';
                }
            }
        
            return $return;
        }
        
//--------------------------------

        function update_resource_link($client, $sesskey, $courseid, $url, $name, $summary, $type, $rname) {
            //error_log(__METHOD__, 1, "schewe@oncampus.de");
            $return = '';
            if ($result = parent::update_resource_link($client, $sesskey, $courseid, $url, $name, $summary, $type, $rname)) {
                if (!empty($result->error)) {
                    $return = 'error';
                } 
                else 
                {
                    $return = 'success';
                }
            }
        
            return $return;
        }
        
//------------------------------------------------------------------

    /**
     * Sends an error response back to the client.
     * 
     * @param string $msg The error message to return.
     * @return An error message string.
     */
        function error($msg) {
            return new soap_fault('Client', '', $msg);
        }

    }

?>