<?php // $Id: server.class.php,v 1.5 2008/02/15 09:47:12 schewe Exp $

/**
 * Base class for web services server layer.
 *
 * @package Web Services
 * @version $Id: server.class.php,v 1.5 2008/02/15 09:47:12 schewe Exp $
 * @author Open Knowledge Technologies - http://www.oktech.ca/
 * @author Justin Filip <jfilip@oktech.ca>
 * @author Kay Schewe <schewe@oncampus.de>
 * @author Dirk Winning <winning@oncampus.de>
 * @author Marc Vorreiter <vorreiter@oncampus.de>
 *  
	 function server() {
        function upgrade($oldversion) {
        function init($client) {
        function add_session_key() {
        function get_session_key($client) {
        function get_session_user($client) {
        function validate_client($client = 0, $sesskey = '') {
        function validate_session($client, $request) {
        function validate_request($request) {
        function login($username, $password) {
        function logout($client, $sesskey) {
        function client_disconnect($client) {
        function edit_user($client, $sesskey, $users) {
        function get_user($client, $sesskey, $userids, $idfield = 'idnumber') {
        function edit_course($client, $sesskey, $courses) {
        function get_course($client, $sesskey, $courseids, $idfield = 'idnumber') {
        function get_grades($client, $sesskey, $userid, $courseids, $idfield = 'idnumber') {
        function enrol_students($client, $sesskey, $courseid, $userids, $idfield = 'idnumber') {
        function enrol($client, $sesskey, $courseid, $userids , $roleid ) {
        function unenrol($client, $sesskey, $courseid, $userids) {
        function import_backup($client, $sesskey, $courseid, $file)
        function add_to_metacourse($client, $sesskey, $courseid1, $courseid2)
        function add_resource_link($client, $sesskey, $courseid, $url, $name, $summary = '', $type = 'file')
        function update_resource_link($client, $sesskey, $courseid, $url, $name, $summary = '', $type = 'repository', $rname = 'Online Modul starten')
        function error($msg) {
        function isadmin($userid) {
        function isteacher($courseid, $userid, $includeadmin = true) {
        function isteacherinanycourse($userid, $includeadmin = true) {
        function isteacheredit($courseid, $userid) {
        function iscreator($userid) {
        function debug_output($output)
*/


    require_once('../config.php');

	// AC write-in-DB-class
	require_once($CFG->dirroot . '/AC/acifwrite.php');

    define('DEBUG', true);


/**
 * The main server class.
 * 
 * This class is broken up into three main sections of methods:
 * 1. Methods that perform actions related to client requests.
 * 2. Methods that handle server setup, incoming client requests, and returning a
 *    response to the client.
 * 3. Utility functions that perform functions such as datetime format conversion or
 *    replication of Moodle library functions in a manner safe for usage within this
 *    web services implementatation.
 * 
 * The only methods that need to be extended in a child class are main() and any of
 * the service methods which need special transport-protocol specific handling of
 * input and / or output data.
 * 
 * 
 * @package Web Services
 * @author Open Knowledge Technologies - http://www.oktech.ca/
 * @author Justin Filip <jfilip@oktech.ca>
 */
    class server {

        var $version        = 2006050800;
        var $sessiontimeout = 1800;  // 30 minutes.
        var $using17;

    /**
     * Constructor method.
     * 
     * @uses $CFG
     * @param none
     * @return none
     */
        function server() {
            global $CFG;

            if (DEBUG) $this->debug_output("Server init...");

            $this->using17 = file_exists($CFG->libdir . '/accesslib.php');

        /// Check for any upgrades.
            if (empty($CFG->webservices_version)) {
                $this->upgrade(0);
            } else if ($CFG->webservices_version < $this->version) {
                $this->upgrade($CFG->webservices_version);
            }
        }


    /**
     * Performs an upgrade of the webservices system.
     * 
     * @uses $CFG
     * @param int $oldversion The old version number we are upgrading from.
     * @return boolean True if successful, False otherwise.
     */
        function upgrade($oldversion) {
            global $CFG;

            if (DEBUG) $this->debug_output('Starting WS upgrade to version ' . $oldversion);

            ob_start();

            $return = true;

            if ($this->using17){
                require_once($CFG->libdir . '/ddllib.php');

                $return = install_from_xmldb_file($CFG->dirroot . '/ws/db/install.xml');

            } else {
                if ($oldversion < 2006050800) {
                    if ($CFG->dbtype == 'mysql') {
                        if ($return) {
                            $return = modify_database('', "
                                CREATE TABLE `prefix_webservices_clients_allow` (
                                    `id` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
                                    `client` VARCHAR(15) NOT NULL DEFAULT '0.0.0.0',
                                    PRIMARY KEY `id` (`id`)
                                );
                            ");
                        }

                        if ($return) {
                            $return = modify_database('', "
                                CREATE TABLE `prefix_webservices_sessions` (
                                    `id` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
                                    `sessionbegin` INT(10) UNSIGNED NOT NULL DEFAULT 0,
                                    `sessionend` INT(10) UNSIGNED NOT NULL DEFAULT 0,
                                    `sessionkey` VARCHAR(32) NOT NULL DEFAULT '',
                                    `userid` INT(10) UNSIGNED NOT NULL DEFAULT 0,
                                    `verified` TINYINT(1) NOT NULL DEFAULT 0,
                                    PRIMARY KEY `id` (`id`)
                                );
                            ");
                        }
                    } else if ($CFG->dbtype == 'postgres7') {
                        if ($return) {
                            $return = modify_database('', "
                                CREATE TABLE prefix_webservices_clients_allow (
                                    id SERIAL PRIMARY KEY,
                                    client VARCHAR(15) NOT NULL DEFAULT '0.0.0.0'
                                );
                            ");
                        }

                        if ($return) {
                            $return = modify_database('', "
                                CREATE TABLE prefix_webservices_sessions (
                                    id SERIAL PRIMARY KEY,
                                    sessionbegin INTEGER NOT NULL DEFAULT 0,
                                    sessionend INTEGER NOT NULL DEFAULT 0,
                                    sessionkey VARCHAR(32) NOT NULL DEFAULT '',
                                    userid INTEGER NOT NULL DEFAULT 0,
                                    verified INTEGER NOT NULL DEFAULT 0
                                );
                            ");
                        }
                    }
                }
            }

            if (ob_get_length() && trim(ob_get_contents())) {
            /// Return an error with  the contents of the output buffer.
                if (DEBUG) $this->debug_output('Database output: ' . trim(ob_get_clean()));
            }

            if ($return) {
                set_config('webservices_version', $this->version);

                if (DEBUG) $this->debug_output('Upgraded from ' . $oldversion . ' to ' . $this->version);
            } else {
                if (DEBUG) $this->debug_output('ERROR: Could not upgrade to version ' . $this->version);
            }

            ob_end_clean();
            
            return $return;
        }


    /**
     * Initializes a connection to a new client by generating a random session
     * key to be used for communications with this specific client.
     * 
     * @param int $client The client session record ID.
     * @return object A new request object containing information the client
     *                needs for further communication or an error object.
     */
        function init($client) {
            if (DEBUG) $this->debug_output('Running INIT for client: ' . $client);

       /// Add this client's database record.
            if (!$sess = get_record('webservices_sessions', 'id', $client)) {
                if (DEBUG) $this->debug_output('No session '. $client);
                return $this->error('Could not get validated client session (' . $client . ').');
            } 

            $sess->sessionbegin = time();
            $sess->sessionend   = 0;
            $sess->sessionkey   = $this->add_session_key();
 
            if (!update_record('webservices_sessions', $sess)) {
                if (DEBUG) $this->debug_output('No update');
                return $this->error('Could not initialize client session (' . $client . ').');
            }

            if (DEBUG) {
                $this->debug_output('Login successful.');
            }

        /// Return standard data to be converted into the appropriate data format
        /// for return to the client.
            return array(
                'client'     => $client,
                'sessionkey' => $sess->sessionkey
            );
        }


    /**
     * Creates a new session key.
     * 
     * @param none
     * @return string A 32 character session key.
     */
        function add_session_key() {
            $time    = (string)time();
            $randstr = (string)random_string(10);

        /// XOR the current time and a random string.
            $str  = $time;
            $str ^= $randstr;

        /// Use the MD5 sum of this random 10 character string as the session key.
            return md5($str);
        }


    /**
     * Gets the session key from the database for a particular client.
     * 
     * @param int $client The client session record ID.
     * @return string|boolean The client's current session key or False.
     */
        function get_session_key($client) {
            if (!$sess = get_record('webservices_sessions', 'id', $client,
                                    'sessionend', 0, 'verified', 1)) {
                if (DEBUG) $this->debug_output('No session exists for client: ' . $client);
                return false;
            }
            
            return $sess->sessionkey;
        }


    /**
     * Get the userid from the database for a particular client's session.
     * 
     * @param int $client the client session record ID.
     * @return int|boolean The client's current userid or False.
     */
        function get_session_user($client) {
            if (!$sess = get_record('webservices_sessions', 'id', $client,
                                    'sessionend', 0, 'verified', 1)) {
                if (DEBUG) $this->debug_output('No session exists for client: ' . $client);
                return false;
            }
            
            return $sess->userid;
        }


    /**
     * Validate's that a client has an existing session.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @return boolean True if the client is valid, False otherwise.
     */
        function validate_client($client = 0, $sesskey = '') {
        /// We can't validate a session that hasn't even been initialized yet.
            if (!$sess = get_record('webservices_sessions', 'id', $client,
                               'sessionend', 0, 'verified', 1)) {
                if (DEBUG) $this->debug_output('no session key for client 1(' . $client . ').');
                return false;
            }

        /// Validate this session.
            if ($sesskey != $sess->sessionkey) {
              if (DEBUG) $this->debug_output('Invalid session key for client (' . $client . ').');
                return false;
            }

            return true;
        }


    /**
     * Validates a client session to determine whether it has expired or not.
     * 
     * @param int $client The client session record ID.
     * @param object $request The request object from the client.
     * @return boolean True if session is valid, False otherwise.
     */
        function validate_session($client, $request) {
            if (!$sess = get_record('webservices_sessions', 'id', $client)) {
              if (DEBUG) $this->debug_output('no session key for client 2(' . $client . ').');
                return false;
            }
            
            if ($sess->sessionkey != $request->get_sessionkey()) {
                if (DEBUG) $this->debug_output('Invalid session key for client (' . $client . ').');
            }
           
            if ((time() - $sess->sessionbegin) > $this->sessiontimeout) {
                if (DEBUG) $this->debug_output('Session (' . $client . ') expired.');
                return false;
            }
            
            return true;
        }


    /**
     * Validate's a client's request.
     * 
     * @param object $request The request object from the client.
     * @return boolean True if the request is valid, False otherwise. 
     */
        function validate_request($request) {
            return $request->validate();
        }


    /**
     * Validates a client's login request.
     * 
     * @uses $CFG
     * @param array $input Input data from the client request object.
     * @return array Return data (client record ID and session key) to be
     *               converted into a specific data format for sending to the
     *               client.
     */
        function login($username, $password) {
            global $CFG;

        /// Use Moodle authentication.
            $user = authenticate_user_login($username, $password);

            if ($user === false) {
                return $this->error('Invalid username and / or password.');
            } else {
            /// Verify that an active session does not already exist for this user.
                $sql = "SELECT s.* FROM {$CFG->prefix}webservices_sessions AS s ".
                "WHERE s.userid = {$user->id} AND ".
                "s.verified = 1 AND s.sessionend != 0 AND ".
                "(" . time() . " - s.sessionbegin) < " . $this->sessiontimeout;
                
                if (record_exists_sql($sql)) {
                    return $this->error('A session already exists for this user (' . $user->id . ')');
                }

            /// Login valid, create a new session record for this client.
                $sess = new stdClass;
                $sess->userid   = $user->id;
                
                $sess->verified = true;
                $sess->id = insert_record('webservices_sessions', $sess);
                if (!$sess ) {
                  return $this->error('error - probably table missing: mdl_webservices_sessions  ???');
                }
                return $this->init($sess->id);
            }
        }


    /**
     * Logs a client out of the system by removing the valid flag from their
     * session record and any user ID that is assosciated with their particular
     * session.
     * 
     * @param integer $client The client record ID.
     * @param string $sesskey The client session key.
     * @return boolean True if successfully logged out, false otherwise.
     */
        function logout($client, $sesskey) {
            if (!$this->validate_client($client, $sesskey)) {
                return $this->error('Invalid client connection.');
            }

            if ($sess = get_record('webservices_sessions', 'id', $client,
                                   'sessionend', 0, 'verified', 1)) {
                $sess->userid   = 0;
                $sess->verified = 0;

                if (update_record('webservices_sessions', $sess)) {
                    return $this->client_disconnect($client);
                } else {
                    return false;
                }
            }
            return false;
        }


    /**
     * Closes a client's session on the system.
     * 
     * @param int $client The client session record ID.
     * @return boolean True on success, False otherwise.
     */
        function client_disconnect($client) {
            if ($sess = get_record('webservices_sessions', 'id', $client,
                                   'sessionend', 0, 'verified', 0)) {
                $sess->sessionend = time();
                
                if (!update_record('webservices_sessions', $sess)) {
                    return false;
                } else {
                    return true;
                }
            }

            return false;
        }


        function get_syncgrade($client, $sesskey) {
            if (!$this->validate_client($client, $sesskey)) {
                return $this->error('Invalid client connection.');
            }
			ob_start();
			$sgrade=get_record('oncampus_sync_grades');
			
			
			if (empty($sgrade)){
				$sgrade=new object;
				$sgrade->id=0;
				$sgrade->user='';
				$sgrade->course='';
				$sgrade->grade=0;
				$sgrade->gradetime=0;
			}
			ob_end_clean();

            return $sgrade;        }
	
        function del_syncgrade($client, $sesskey, $id) {
            if (!$this->validate_client($client, $sesskey)) {
                return $this->error('Invalid client connection.');
            }
			
			$delresponse='';
			ob_start();
			if (!delete_records_select("oncampus_sync_grades","id=".$id)){
			$delresponse='error deleting syncgrade';
			 }
            
			ob_end_clean();

            return $delresponse;        }	
		
		
		
    /**
     * Edit user records (add/update/delete).
     * 
     * @uses $CFG
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param array $users An array of user records (objects or arrays) for editing
     *                     (including opertaion to perform).
     * @return array Return data (user record) to be converted into a
     *               specific data format for sending to the client.
     */
        function edit_user($client, $sesskey, $users) {
            global $CFG;

            if (DEBUG) $this->debug_output('Trying to edit a user (Add, Update, Delete).');
            
            if (!$this->validate_client($client, $sesskey)) {
                return $this->error('Invalid client connection.');
            }

            if (DEBUG) $this->debug_output('Trying to edit a user (client connection valid).');
            
        /// Verify that the user for this session can perform this operation.
        	if (DEBUG) $this->debug_output('Trying to edit a user (uid:'.$this->get_session_user($client).').');
        	
            $uid = $this->get_session_user($client);

            if (DEBUG) $this->debug_output('Trying to edit a user (isadmin:'.$this->isadmin($uid).').');
            
            if (!$this->isadmin($uid)) {
                return $this->error('You do not have proper access to perform this operation.');
            }

            if (DEBUG) $this->debug_output('Trying to edit a user ('.var_export($users, true).').');
            
            $rusers = array();

            if (!empty($users['users'])) {
            	
            	if (DEBUG) $this->debug_output('Trying to edit a user ('.count($users).').');
            	
                foreach ($users['users'] as $ue) {
                /// If the input was an array, convert it into an object.
                    if (is_array($ue)) {
                        $keys = array_keys($ue);
                        $vals = array_values($ue);

                        $ua = new stdClass;

                        while ($key = current($keys)) {
                            $ua->$key = current($vals);

                            next($keys);
                            next($vals);
                        }

                        $user = $ua;
                    } else {
                        $user = $ue;
                    }
                    
                    if (DEBUG) $this->debug_output('Trying to edit a user ('.$user->action.').');

                    $ruser = new stdClass;
                    
                    $user->firstname = addslashes($user->firstname);//ocmod add
            	      $user->lastname = addslashes($user->lastname);//ocmod add
            	      if (DEBUG) $this->debug_output('User name escaped: '.$user->firstname." ".$user->lastname);
            	      
                    switch ($user->action) {

// ++++++++++++++++++++++++++++++++++++++++++  

                        case 'Add':
                            $useradd = $user;
                            unset($useradd->action);
							$usercustomerrole = $useradd->customerrole;
                            unset($useradd->customerrole);
							$userbirthdate = $useradd->birthdate;
                            unset($useradd->birthdate);
							$usercompany = $useradd->company;
                            unset($useradd->company);							
							
                        /// This database operation MIGHT throw an HTML error message,
                        /// so we've got to catch that and send it back in an error
                        /// request.
                            ob_start();

                            if (!isset($useradd->confirmed) || empty($useradd->confirmed)) {
                                $useradd->confirmed = true;
                            }

			                      if (!isset($useradd->mnethostid) || empty($useradd->mnethostid)) {                                
				                        $useradd->mnethostid = 1;                            
			                      }

                            $useradd->id = insert_record('user', $useradd);

                            if (ob_get_length() && trim(ob_get_contents())) {
                            /// Return an error with  the contents of the output buffer.
                                $msg = trim(ob_get_clean());
                                return $this->error('Database error: ' . $msg);
                            }
                            ob_end_clean();
                            
                            if (empty($useradd->id)) {
                                $ruser->error = 'Could not add user: ' . fullname($useradd);
                            } else {
                                	$ruser = get_record('user', 'id', $useradd->id);
                        					// Add AC-User
                        					if (!aiw_Create_user($useradd->username, $useradd->firstname, $useradd->lastname)) {
                        						$ruser->error = 'AC_error Create_user: ' . fullname($useradd);
                        					}
											
											// Bei B2B-Benutzern Profilfeld Kundenrolle setzen
											if ($usercustomerrole=='Business'){
									            $uifrecord = new object();
									            $uifrecord->userid = $useradd->id;
												// Profilfeld fr Kundenrolle ermitteln
												$uifrecord->fieldid = get_field_select('user_info_field', 'id', "shortname = 'customerrole'");
											    $uifrecord->data = 'Business';
												
									            if (!insert_record('user_info_data', $uifrecord)) {
													if (DEBUG) $this->debug_output('Error adding user info data: ' . var_dump($uifrecord));
									                error("Database error - Couldn't add user info field.", '', true);
									            }
												else{
													// nach erfolgreichem Anlegen des Profileintrags: Eintrag in Log-Tabelle 
													add_to_log(1, "user_info_data", "insert", "", "customerrole=Business", "", $user->id);
												}
												
												// Birthdate
									            $uifrecord = new object();
									            $uifrecord->userid = $useradd->id;
												// Profilfeld fr Geburtsdatum ermitteln
												$uifrecord->fieldid = get_field_select('user_info_field', 'id', "shortname = 'birthdate'");
											    $uifrecord->data = $userbirthdate;
												
									            if (!insert_record('user_info_data', $uifrecord)) {
													if (DEBUG) $this->debug_output('Error adding user info data: ' . var_dump($uifrecord));
									                error("Database error - Couldn't add user info field.", '', true);
									            }
												else{
													// nach erfolgreichem Anlegen des Profileintrags: Eintrag in Log-Tabelle 
													add_to_log(1, "user_info_data", "insert", "", "birthdate", "", $user->id);
												}
												
												// Company
									            $uifrecord = new object();
									            $uifrecord->userid = $useradd->id;
												// Profilfeld fr Firma ermitteln
												$uifrecord->fieldid = get_field_select('user_info_field', 'id', "shortname = 'company'");
											    $uifrecord->data = $usercompany;
												
									            if (!insert_record('user_info_data', $uifrecord)) {
													if (DEBUG) $this->debug_output('Error adding user info data: ' . var_dump($uifrecord));
									                error("Database error - Couldn't add user info field.", '', true);
									            }
												else{
													// nach erfolgreichem Anlegen des Profileintrags: Eintrag in Log-Tabelle 
													add_to_log(1, "user_info_data", "insert", "", "company", "", $user->id);
												}

												
											} // endif business
                            }
                            break;
// ++++++++++++++++++++++++++++++++++++++++++  

                        case 'Update':
                            $userup = $user;
                            $uid    = $userup->idnumber;
                            $dbfail = false;
                            unset($userup->action);
							$usercustomerrole = $userup->customerrole;
							unset($userup->customerrole);
							$userbirthdate = $userup->birthdate;
                            unset($userup->birthdate);
							$usercompany = $userup->company;
                            unset($userup->company);

                            if (DEBUG) $this->debug_output('Attempting to update user ID: ' . $uid);

                        /// This database operation MIGHT throw an HTML error message,
                        /// so we've got to catch that and send it back in an error
                        /// request.
                            ob_start();

                            $user = get_record('user', 'idnumber', $userup->idnumber);

                            if (ob_get_length() && trim(ob_get_contents())) {
                            /// Return an error with  the contents of the output buffer.
                                $msg = trim(ob_get_clean());
                                $ruser = $user;
                                $ruser->error = 'Database error: ' . $msg;
                                $dbfail       = true;
                            }
                            ob_end_clean();

                            if (!$dbfail && empty($user)) {
                                $ruser = $user;
                                $ruser->error = 'Could not find user ID: ' . $uid;
                            } else {
                            /// Update values in the $user database record with what
                            /// the client supplied.
                                if (isset($userup->confirmed)) {
                                    $user->confirmed = $userup->confirmed;
                                }
                                if (isset($userup->policyagreed)) {
                                    $user->policyagreed = $userup->policyagreed;
                                }
                                if (isset($userup->deleted)) {
                                    $user->deleted = $userup->deleted;
                                }
                                if (isset($userup->username)) {
                                    $user->username = $userup->username;
                                }
                                if (isset($userup->password)) {
                                    $user->password = $userup->password;
                                }
                                if (isset($userup->firstname)) {
                                    $user->firstname = $userup->firstname;
                                }
                                if (isset($userup->lastname)) {
                                    $user->lastname = $userup->lastname;
                                }
                                if (isset($userup->email)) {
                                    $user->email = $userup->email;
                                }
                                if (isset($userup->emailstop)) {
                                    $user->emailstop = $userup->emailstop;
                                }
                                if (isset($userup->icq)) {
                                    $user->icq = $userup->icq;
                                }
                                if (isset($userup->skype)) {
                                    $user->skype = $userup->skype;
                                }
                                if (isset($userup->yahoo)) {
                                    $user->yahoo = $userup->yahoo;
                                }
                                if (isset($userup->aim)) {
                                    $user->aim = $userup->aim;
                                }
                                if (isset($userup->msn)) {
                                    $user->msn = $userup->msn;
                                }
                                if (isset($userup->phone1)) {
                                    $user->phone1 = $userup->phone1;
                                }
                                if (isset($userup->phone2)) {
                                    $user->phone2 = $userup->phone2;
                                }
                                if (isset($userup->institution)) {
                                    $user->institution = $userup->institution;
                                }
                                if (isset($userup->department)) {
                                    $user->department = $userup->department;
                                }
                                if (isset($userup->address)) {
                                    $user->address = $userup->address;
                                }
                                if (isset($userup->city)) {
                                    $user->city = $userup->city;
                                }
                                if (isset($userup->country)) {
                                    $user->country = $userup->country;
                                }
                                if (isset($userup->lang)) {
                                    $user->lang = $userup->lang;
                                }
                                if (isset($userup->theme)) {
                                    $user->theme = $userup->theme;
                                }   
                                if (isset($userup->timezone)) {
                                    $user->timezone = $userup->timezone;
                                }
                                if (isset($userup->emailstop)) {
                                    $user->emailstop = $userup->emailstop;
                                }
                                if (isset($userup->firstaccess)) {
                                    $user->firstaccess = $userup->firstaccess;
                                }
                                if (isset($userup->lastaccess)) {
                                    $user->lastaccess = $userup->lastaccess;
                                }
                                if (isset($userup->lastlogin)) {
                                    $user->lastlogin = $userup->lastlogin;
                                }
                                if (isset($userup->currentlogin)) {
                                    $user->currentlogin = $userup->currentlogin;
                                }
                                if (isset($userup->lastIP)) {
                                    $user->lastIP = $userup->lastIP;
                                }
                                if (isset($userup->secret)) {
                                    $user->secret = $userup->secret;
                                }
                                if (isset($userup->url)) {
                                    $user->url = $userup->url;
                                }
                                if (isset($userup->description)) {
                                    $user->description = $userup->description;
                                }
				//$user->description = addslashes($user->description);
				unset($user->description);
                                if (isset($userup->mailformat)) {
                                    $user->mailformat = $userup->mailformat;
                                }
                                if (isset($userup->maildigest)) {
                                    $user->maildigest = $userup->maildigest;
                                }
                                if (isset($userup->maildisplay)) {
                                    $user->maildisplay = $userup->maildisplay;
                                }
                                if (isset($userup->htmleditor)) {
                                    $user->htmleditor = $userup->htmleditor;
                                }
                                if (isset($userup->autosubscribe)) {
                                    $user->autosubscribe = $userup->autosubscribe;
                                }
				     unset($user->imagealt);                                
                                $user->timemodified = time();

                            /// This database operation MIGHT throw an HTML error message,
                            /// so we've got to catch that and send it back in an error
                            /// request.
                                ob_start();

                                $success = update_record('user', $user);

                                if (ob_get_length() && trim(ob_get_contents())) {
                                /// Return an error with  the contents of the output buffer.
                                    $msg = trim(ob_get_clean());
                                    
                                    $ruser = $user;
                                    $ruser->error = 'Database error: ' . $msg;
                                    $dbfail       = true;
                                }
                                ob_end_clean();
                                if (DEBUG) $this->debug_output('Update result for user: ' .(($success)?'success':'failed').': '.var_export($user, true));
                                if ($dbfail || !$success) {
                                    $ruser = $user;
                                    $ruser->error = 'Could not update user: ' . $uid;
                                } 
				     else //if (!$dbfail && $success)
						{
                        $ruser = get_record('user', 'id', $user->id);
						if (DEBUG) $this->debug_output('Ruser: ' .var_export($ruser, true));
						// Mod AC-User
						if ( !aiw_Modify_user($userup->username, $userup->firstname, $userup->lastname)) 
							{$ruser->error = 'AC_error Modify_user: ' . $uid;}
							
						// -----------------------------------------------------
						
						// B2B-Erweiterung
						
						// Kundenrolle
						$fieldid_customerrole = get_field_select('user_info_field', 'id', "shortname = 'customerrole'");
						$uidrecord=get_record_select("user_info_data","fieldid=".$fieldid_customerrole." and userid=".$user->id);
						
						if (empty($uidrecord))
							{
							if (DEBUG) $this->debug_output('noch keine Kundenrolle vorhanden - neue:'.$usercustomerrole.':');
							if ($usercustomerrole=='Business')
								{
								$uidrecord = new object();
								$uidrecord->userid = $user->id;
								$uidrecord->fieldid = $fieldid_customerrole;
								$uidrecord->data = $usercustomerrole;
								
					            if (!insert_record('user_info_data', $uidrecord)) {
									if (DEBUG) $this->debug_output('Error adding user info data: ' . var_dump($uidrecord));
					                error("Database error - Couldn't add user info field.", '', true);
						            }
								else{
									// nach erfolgreichem Anlegen des Profileintrags: Eintrag in Log-Tabelle 
									add_to_log(1, "user_info_data", "insert", "", "customerrole=Business", "", $user->id);
									}
								}
							}
						else
							{
							if (DEBUG) $this->debug_output('update Kundenrolle - neue:'.$usercustomerrole.':');
							$uidrecord->data = $usercustomerrole;
						    if (!update_record('user_info_data', $uidrecord)) {
						    	error("Database error - Couldn't modify the user info data list.", '', true);
								}
							}
						// bisherige Testergebnisse ins Portal bertragen wenn neue Kundenrolle=Business
						
						$quizmoduleid = get_field_select("modules", "id", "name='quiz'");
						if ($usercustomerrole=='Business') {
							
							if($oldgrades = get_records_select("quiz_grades", "userid=".$user->id)){
								
        						foreach ($oldgrades as $oldgrade) {
        							
        							$quizcm=get_record_select("course_modules", "module='".$quizmoduleid."' and instance='".$oldgrade->quiz."'");
									$mingrade=0;
									
									$mingrade= get_field_select("oncampus_zertificat_lock", "mingrade", "quiz='".$quizcm->id."'");									
        							$newgrade=0;
									if (($oldgrade->grade)>=$mingrade)
										{$newgrade=1;}
									$courseidnumber = get_field_select('course', 'idnumber', "id = '".$quizcm->course."'");
									
        							$syncgrade = new object();
									if ($syncgrade = get_record_select("oncampus_sync_grades", "course='".$courseidnumber."' and user='".$user->idnumber."'")){
							       		$syncgrade->grade = $newgrade;
								        $syncgrade->gradetime = time();
							   		    if (!update_record('oncampus_sync_grades', $syncgrade)) {
							            	notify('Could not update best grade in sync');
								            return false;
							    	    }
									} else {
							        	$syncgrade->user = $user->idnumber;
							        	$syncgrade->course = $courseidnumber;
							        	$syncgrade->grade = $newgrade;
							        	$syncgrade->gradetime = time();
							        	if (!insert_record('oncampus_sync_grades', $syncgrade)) {
							           		notify('Could not insert new best grade in sync');
							            	return false;
							        	}
									}
									        							
        						}
        						
        					}
        					
						}
							
						
						// Geburtsdatum
						$fieldid_birthdate = get_field_select('user_info_field', 'id', "shortname = 'birthdate'");
						$uidrecord=get_record_select("user_info_data","fieldid=".$fieldid_birthdate." and userid=".$user->id);
						
						if (empty($uidrecord))
							{
							if (DEBUG) $this->debug_output('noch kein Geburtsdatum vorhanden - neue:'.$userbirthdate.':');
							if ($usercustomerrole=='Business')
								{
								$uidrecord = new object();
								$uidrecord->userid = $user->id;
								$uidrecord->fieldid = $fieldid_birthdate;
								$uidrecord->data = $userbirthdate;
								
					            if (!insert_record('user_info_data', $uidrecord)) {
									if (DEBUG) $this->debug_output('Error adding user info data: ' . var_dump($uidrecord));
					                error("Database error - Couldn't add user info field.", '', true);
						            }
								else{
									// nach erfolgreichem Anlegen des Profileintrags: Eintrag in Log-Tabelle 
									add_to_log(1, "user_info_data", "insert", "", "birthdate", "", $user->id);
									}
								}
							}
						else
							{
							if (DEBUG) $this->debug_output('update birthdate - neue:'.$userbirthdate.':');
							$uidrecord->data = $userbirthdate;
						    if (!update_record('user_info_data', $uidrecord)) {
						    	error("Database error - Couldn't modify the user info data list.", '', true);
								}
							}
						
						// Firma
						$fieldid_company = get_field_select('user_info_field', 'id', "shortname = 'company'");
						$uidrecord=get_record_select("user_info_data","fieldid=".$fieldid_company." and userid=".$user->id);
						
						if (empty($uidrecord))
							{
							if ($usercustomerrole=='Business')
								{
								$uidrecord = new object();
								$uidrecord->userid = $user->id;
								$uidrecord->fieldid = $fieldid_company;
								$uidrecord->data = $usercompany;
								
					            if (!insert_record('user_info_data', $uidrecord)) {
									if (DEBUG) $this->debug_output('Error adding user info data: ' . var_dump($uidrecord));
					                error("Database error - Couldn't add user info field.", '', true);
						            }
								else{
									// nach erfolgreichem Anlegen des Profileintrags: Eintrag in Log-Tabelle 
									add_to_log(1, "user_info_data", "insert", "", "company", "", $user->id);
									}
								}
							}
						else
							{
							$uidrecord->data = $usercompany;
						    if (!update_record('user_info_data', $uidrecord)) {
						    	error("Database error - Couldn't modify the user info data list.", '', true);
								}
							}
						
						// -----------------------------------------------------
                        }
                }

                        break;

// ++++++++++++++++++++++++++++++++++++++++++  

                        case 'Delete':
                            $uid    = $user->idnumber;
                            $dbfail = false;

                        /// Deleting an existing user.
                            if (DEBUG) $this->debug_output('Attempting to delete user ID: ' . $uid);
                            
                        /// This database operation MIGHT throw an HTML error message,
                        /// so we've got to catch that and send it back in an error
                        /// request.
                            ob_start();

                            $user = get_record('user', 'idnumber', $uid);

                            if (ob_get_length() && trim(ob_get_contents())) {
                            /// Return an error with  the contents of the output buffer.
                                $msg = trim(ob_get_clean());
                                
                                $ruser->error = 'Database error: ' . $msg;
                                $dbfail       = true;
                            }
                            ob_end_clean();

                            if (!$dbfail && empty($user)) {
                                $ruser->error = 'Could not find user ID: ' . $uid;
                            } else {
                            /// 'Delete' the user the Moodle way.
                                $updateuser = new stdClass;
                                $updateuser->id = $user->id;
                                $updateuser->deleted = "1";
                                $updateuser->username = "$user->email.".time();  // Remember it just in case
                                $updateuser->email = "";                         // Clear this field to free it up
                                $updateuser->idnumber = "";                      // Clear this field to free it up
                                $updateuser->timemodified = time();
                                
                            /// This database operation MIGHT throw an HTML error message,
                            /// so we've got to catch that and send it back in an error
                            /// request.
                                ob_start();

                                $success = update_record('user', $updateuser);

                                if (ob_get_length() && trim(ob_get_contents())) {
                                /// Return an error with  the contents of the output buffer.
                                    $msg = trim(ob_get_clean());
                                    
                                    $ruser->error = 'Database error.' . $msg;
                                    $dbfail       = true;
                                }
                                ob_end_clean();
                                
                                if (!$dbfail && $success) {
                                    if ($this->using17) {
                                        delete_records('role_assignments', 'userid', $user->id);
					//role_unassign(0, $user->id, 0, 0);
                                    } else {
                                        unenrol_student($user->id);  // From all courses
                                        remove_teacher($user->id);   // From all courses
                                        remove_admin($user->id);
                                    }
                                    
                                    $ruser = get_record('user', 'id', $user->id);
					// Remove AC-User
					if (!aiw_Delete_user($user->username)) {
						$ruser->error = 'AC_error Delete_user: ' . $uid;
					}
                                } else if ($dbfail || !$success){
                                    $ruser        = $user;
                                    $ruser->error = 'Could not delete user ID: ' . $uid;
                                }                                    
                            }

                            break;

// ++++++++++++++++++++++++++++++++++++++++++  

                    }

                    $rusers[] = $ruser;
                }
            }

            return $rusers;
        }


    /**
     * Find and return a list of user records.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param array $userids An array of input user idnumber values.
     * @param string $idfield The field used to compare the user ID fields against.
     * @return array Return data (user record) to be converted into a
     *               specific data format for sending to the client.
     */
        function get_user($client, $sesskey, $userids, $idfield = 'idnumber') {
            if (!$this->validate_client($client, $sesskey)) {
                return $this->error('Invalid client connection.');
            }

        /// Verify that the user for this session can perform this operation.
            $uid = $this->get_session_user($client);

            if (!$this->isteacherinanycourse($uid)) {
                return array(
                    'error' => 'You do not have proper access to perform this operation.',
                    'stat'  => WS_STAT_E_USERID
                );
            }

            $ret = array();  // Return array.

            if (empty($userids)) {
                return $this->error('No user data supplied.');
            }

            foreach ($userids as $userid) {
                $error = '';

            /// This database operation MIGHT throw an HTML error message,
            /// so we've got to catch that and send it back in an error
            /// request.
                ob_start();

                $user = get_record('user', $idfield, $userid);

                if (ob_get_length() && trim(og_get_contents())) {
                /// Return an error with  the contents of the output buffer.
                    $msg = trim(ob_get_clean());

                    $error = 'Database error: ' . $msg;
                }
                ob_end_clean();

                if (!empty($error)) {
                    return $this->error($error);
                }

                if (empty($user)) {
                    $user->error = 'Invalid user ID';
                }

            /// Return fetch users or errors.
                $ret[] = $user;
            }

            return $ret;
        }


    /**
     * Edit user records (add/update/delete).
     * 
     * @uses $CFG
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param array $courses An array of course records (objects or arrays) for editing
     *                       (including opertaion to perform).
     * @return array Return data (course record) to be converted into a specific
     *               data format for sending to the client.
     */
        function edit_course($client, $sesskey, $courses) {
            global $CFG;

            if (!$this->validate_client($client, $sesskey)) {
                return $this->error('Invalid client connection.');
            }

            $uid  = $this->get_session_user($client);
            $site = get_site();
            $ret  = array();

            if (!empty($courses['courses'])) {
                foreach ($courses['courses'] as $ce) {
                /// If the input was an array, convert it into an object.
                    if (is_array($ce)) {
                        $keys = array_keys($ce);
                        $vals = array_values($ce);

                        $ca = new stdClass;

                        while ($key = current($keys)) {
                            $ca->$key = current($vals);

                            next($keys);
                            next($vals);
                        }

                        $course = $ca;
                    } else {
                        $course = $ce;
                    }

                    $rcourse = new stdClass;

                    switch ($course->action) {

// ++++++++++++++++++++++++++++++++++++++++++

                        case 'Add':
                        /// Adding a new course.
                            $courseadd = $course;

                            if (DEBUG) $this->debug_output('Trying to add a new course.');

                        /// Check for correct permissions.
                            if (!$this->iscreator($uid)) {
                                if (DEBUG) $this->debug_output('Invalid access UID: ' . $uid);
                                $rcourse->error = 'You do not have proper access to perform this operation.';

                            } else if (record_exists('course', 'idnumber', $courseadd->idnumber)) {
                                $rcourse->error = 'A course with this ID number already exists: ' .
                                                  $courseadd->idnumber;

                            } else {
                            /// These database operations MIGHT throw an HTML error message,
                            /// so we've got to catch that and send it back in an error
                            /// request.
                                ob_start();
    
                                // place at beginning of category
                                fix_course_sortorder();
                                $courseadd->sortorder = get_field_sql("SELECT min(sortorder)-1 FROM " .
                                    "{$CFG->prefix}course WHERE category=$courseadd->category");                
                                if (empty($courseadd->sortorder)) {
                                    $courseadd->sortorder = 100;
                                }
    
                                if (!isset($courseadd->maxbytes)) {
                                    $byteschoices = get_max_upload_sizes($CFG->maxbytes);
                                    $courseadd->maxbytes = key($byteschoices);
                                }
    
                                if (!isset($courseadd->startdate)) {
                                    $courseadd->startdate = time();
                                }
    
                                if (isset($courseadd->categoryname)) //< added by ks
                                {
                                  if (!$category = get_record("course_categories", "name", $courseadd->categoryname)) {
							                        if (DEBUG) $this->debug_output('Category not known, trying to create one!');
                                		  //$tempcat = new stdClass();
									   	                $tempcat->name = $courseadd->categoryname;//get_string('miscellaneous');
              							        	if (!$tempcat->id = insert_record('course_categories', $tempcat)) {
              							            	if (DEBUG) $this->debug_output('Serious error: Could not create category: '.$tempcat->name);
              							            	//$courseadd->category = 1;//< use default category or unset($courseadd->category)
              							        	}
              							        	else
              							        	{
              							        		if (DEBUG) $this->debug_output('Created category: '.$category->id);
              							        		$courseadd->category = $tempcat->id;
              							        	}
                                	}
                                	else
                                	{
                                		if (DEBUG) $this->debug_output('Found category: '.$category->id);
                                		$courseadd->category = $category->id;
                                	}
                                	
                                	unset($courseadd->categoryname);
                                }
                                else 
                                {
                                	/*if (0 == $courseadd->category)
                                	{
                                		$courseadd->category = 1;
                                	}*/
                                	if (DEBUG) $this->debug_output('Category: '.$courseadd->category);
                                }
                                
                                $courseadd->timecreated  = time();
                                $courseadd->timemodified = time();
    
                                $courseadd->id = insert_record('course', $courseadd);
    
                                if (ob_get_length() && trim(ob_get_contents())) {
                                /// Return an error with  the contents of the output buffer.
                                    $msg            = trim(ob_get_contents());
                                    $rcourse->error = 'Database error: ' . $msg;
                                }
                                ob_end_clean();
                                
                                if (empty($courseadd->id)) {
                                    $rcourse->error = 'Could not add course: ' .
                                                        $courseadd->shortname;
                                } else {
                                    require_once($CFG->libdir . '/pagelib.php');
                                    require_once($CFG->libdir . '/blocklib.php');
    
                                /// These API calls MIGHT throw an HTML error message,
                                /// so we've got to catch that and send it back in an error
                                /// request.
                                    ob_start();
    
                                /// Setup page blocks.
                                    $page = page_create_object(PAGE_COURSE_VIEW, $courseadd->id);
                                    blocks_repopulate_page($page);
    
                                    if (ob_get_length() && trim(ob_get_contents())) {
                                        $msg = trim(ob_get_clean());
                                        $rcourse->error = 'API call error: ' . $msg;
                                    }
                                    
                                /// Create a default section.
                                    $section = NULL;
                                    $section->course = $courseadd->id;
                                    $section->section = 0;
                                    $section->id = insert_record('course_sections', $section);
                                    
    
                                    if (ob_get_length() && trim(ob_get_contents())) {
                                        $msg            = trim(ob_get_clean());
                                        $rcourse->error = 'Database error: ' . $msg;
                                    }
                                    ob_end_clean();
    
                                    $rcourse = get_record('course', 'id', $courseadd->id);
					// Add AC-Room
					if (!aiw_Create_room($courseadd->idnumber)) {
						$rcourse->error = 'AC-Error Create_room: ' . $courseadd->idnumber;
					}
					// resort
                                	if (0 < $courseadd->category)
                                	{
	                                	if ($resort_course_array = get_courses($courseadd->category, "fullname ASC", 'c.id,c.fullname,c.sortorder')) {
							 // move it off the range
							 $resort_count = get_record_sql('SELECT MAX(sortorder) AS max, 1
							            FROM ' . $CFG->prefix . 'course WHERE category=' . $courseadd->category);
							 $resort_count = $resort_count->max + 100;
							 begin_sql();
							 foreach ($resort_course_array as $resort_course) {
							         set_field('course', 'sortorder', $resort_count, 'id', $resort_course->id);
							         $resort_count++;
							 }
							 commit_sql();
							 fix_course_sortorder($courseadd->category);
						}
                                	}
                                }
                            }

                            break;

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++

                        case 'Update':
                        /// Updating an existing course.
                            $courseup = $course;
                            $cid      = $courseup->idnumber;
                            $dbfail   = false;
                            
                            if (DEBUG) $this->debug_output('Attempting to update course ID: ' . $cid);
                            
                        /// This database operation MIGHT throw an HTML error message,
                        /// so we've got to catch that and send it back in an error
                        /// request.
                            ob_start();

                            $course = get_record('course', 'idnumber', $cid);

                            if (ob_get_length() && trim(ob_get_contents())) {
                            /// Return an error with  the contents of the output buffer.
                                if (DEBUG) $this->debug_output('E_DB: "' . ob_get_contents() . '"');
                                $msg            = trim(ob_get_clean());
                                $rcourse->error = 'Database error: ' . $msg;
                                $dbfail         = true;
                            }
                            ob_end_clean();

                        /// Check for correct permissions.
                            if (!$dbfail && empty($course)) {
                                $rcourse->error = 'Could not find course ID: ' . $cid;

                            } else if (!$dbfail && !$this->isteacher($course->id, $uid)) {
                                $rcourse->error  = 'You do not have proper access to perform this operation.';

                            } else if (!$dbfail){
                            /// Update values in the course database record with what
                            /// the client supplied.
                                if (isset($courseup->sortorder)) {
                                    $course->sortorder = $courseup->sortorder;
                                }
                                if (isset($courseup->password)) {
                                    $course->password = $courseup->password;
                                }
                                if (isset($courseup->shortname)) {
                                    $course->shortname = $courseup->shortname;
                                }
                                if (isset($courseup->showgrades)) {
                                    $course->showgrades = $courseup->showgrades;
                                }
                                if (isset($courseup->modinfo)) {
                                    $course->modinfo = $courseup->modinfo;
                                }
                                if (isset($courseup->newsitems)) {
                                    $course->newsitems = $course->newsitems;
                                }
                                if (isset($courseup->teacher)) {
                                    $course->teacher = $courseup->teacher;
                                }
                                if (isset($courseup->teachers)) {
                                    $course->teachers = $courseup->teachers;
                                }
                                if (isset($courseup->student)) {
                                    $course->student = $courseup->student;
                                }
                                if (isset($courseup->students)) {
                                    $course->students = $courseup->students;
                                }
                                if (isset($courseup->guest)) {
                                    $course->guest = $courseup->guest;
                                }
                                if (isset($courseup->enrolperiod)) {
                                    $course->enrolperiod = $courseup->enrolperiod;
                                }
                                if (isset($courseup->numsections)) {
                                    $course->numsections = $courseup->numsections;
                                }
                                if (isset($courseup->marker)) {
                                    $course->marker = $courseup->marker;
                                }
                                if (isset($courseup->maxbytes)) {
                                    $course->maxbytes = $courseup->maxbytes;
                                }
                                if (isset($courseup->showreports)) {
                                    $course->showreports = $courseup->showreports;
                                }
                                if (isset($courseup->visible)) {
                                    $course->visible = $courseup->visible;
                                }
                                if (isset($courseup->hiddensections)) {
                                    $course->hiddensections = $courseup->hiddensections;
                                }
                                if (isset($courseup->groupmode)) {
                                    $course->groupmode = $courseup->groupmode;
                                }
                                if (isset($courseup->groupmodeforce)) {
                                    $course->groupmodeforce = $courseup->groupmodeforce;
                                } 
                                if (isset($courseup->lang)) {
                                    $course->lang = $courseup->lang;
                                }
                                if (isset($courseup->cost)) {
                                    $course->cost = $courseup->cost;
                                }
                                if (isset($courseup->timemodified)) {
                                    $course->timemodified = $courseup->timemodified;
                                }
                                if (isset($courseup->metacourse)) {
                                    $course->metacourse = $courseup->metacourse;
                                }
                                if (isset($courseup->startdate)) {
                                    $course->startdate = $courseup->startdate;
                                }
                                if (isset($courseup->fullname)) {
                                    $course->fullname = $courseup->fullname;
                                }
                                if (isset($courseup->summary)) {
                                    $course->summary = $courseup->summary;
                                }
                                if (isset($courseup->format)) {
                                    $course->format = $courseup->format;
                                }
                                if (isset($courseup->idnumber)) {
                                    $course->idnumber = $courseup->idnumber;
                                }
                                if (isset($courseup->category)) {
                                    $course->category = $courseup->category;
                                }

                                ob_start();
                                
                                $success = update_record('course', $course);
                                
                                if (ob_get_length() && trim(ob_get_contents())) {
                                    $msg            = trim(ob_get_clean());
                                    $rcourse->error = 'Database error: ' . $msg;
                                    $dbfail         = true;
                                }
                                ob_end_clean();
                                
                                if (!$dbfail && !$success) {
                                    $rcourse->error = 'Could not update course: ' . $cid;
                                } else if (!$dbfail && $success) {
                                    $rcourse = get_record('course', 'id', $course->id);
                                }
                            }

                            break;

//+++++++++++++++++++++++++++++++++++++++++++++++

                        case 'Delete':
                        /// Deleting an existing course.
                            $cid    = $course->idnumber;
                            $dbfail = false;

                        /// Check for correct permissions.
                            if (!$this->isadmin($uid)) {
                                $rcourse->error = 'You do not have proper access to ' .
                                                  'perform this operation.';
                            } else {
                                if (DEBUG) $this->debug_output('Attempting to delete course ID: ' . $cid);
                                
                            /// This database operation MIGHT throw an HTML error message,
                            /// so we've got to catch that and send it back in an error
                            /// request.
                                ob_start();
    
                                $course = get_record('course', 'idnumber', $cid);
    
                                if (ob_get_length() && trim(ob_get_contents())) {
                                /// Return an error with  the contents of the output buffer.
                                    $msg            = trim(ob_get_clean());
                                    $rcourse->error = 'Database error: ' . $msg;
                                    $dbfail         = true;
                                }
                                ob_end_clean();
    
                                if (!$dbfail && empty($course)) {
                                    $rcourse->error = 'Could not find course ID: ' . $cid;

                                } else if (!$dbfail) {
                                /// 'Delete' the course the Moodle way.
                                    $success_r = true;
                                    $success_d = true;
                                    $success_f = true;
                                    
                                /// These operations MIGHT throw an HTML error message,
                                /// so we've got to catch that and send it back in an error
                                /// request.
                                    ob_start();
    
                                    require_once($CFG->libdir . '/moodlelib.php');
                                    $success_r = remove_course_contents($course->id, false);
                                    
                                    if (ob_get_length() && trim(ob_get_contents())) {
                                    /// Return an error with  the contents of the output buffer.
                                        $msg  = trim(ob_get_clean());
                                        $rcourse->error = 'API call error: ' . $msg;
                                    }
                                
                                    if ($success_r && !isset($rcourse->error)) {
                                        $success_d = delete_records('course', 'id', $course->id);
                                    } 
                                
                                    if (ob_get_length() && trim(ob_get_contents())) {
                                    /// Return an error with a E_DB status set and the
                                    /// contents of the output buffer.
                                        $msg = trim(ob_get_clean());
                                        $rcourse->error('Database error.', WS_STAT_E_DB, $msg);
                                    }
                                
                                    if ($success_r && $success_d && !isset($rcourse->error) &&
                                        ($dir = @opendir($CFG->dataroot . '/' . $course->id))) {
                                        closedir($dir);
                                        require_once($CFG->libdir . '/filelib.php');
                                        $success_f = fulldelete($CFG->dataroot . '/' . $course->id);
                                    }
    
                                    if (ob_get_length() && trim(ob_get_contents())) {
                                    /// Return an error with  the contents of the output buffer.
                                        $msg = trim(ob_get_clean());
                                        return $this->error('API call error (2).', WS_STAT_E_API, $msg);
                                    }
    
                                    ob_end_clean();

                                    if (!isset($rcourse->error)) {
                                        if (!$success_r) {
                                            $rcourse->error = 'Error deleting some of the course contents (' .
                                                              $cid . ').';
                                        } else if (!$success_d) {
                                            $rcourse->error = 'Error deleting the course record (' . $cid . ').';
                                        } else if (!$success_f) {
                                            $rcourse->error = 'Error deleting the course data (' . $cid . ').';
                                        } else {
						// Remove AC-Room
						if (!aiw_Delete_room($cid)) {
								$rcourse->error = 'AC_error Delete_room: ' . $cid;
						}
                                            $rcourse = $course;
                                        }
                                    }
                                }
                            }

                            break;

//+++++++++++++++++++++++++++++++++++++++++++++++

                    }

                    $ret[] = $rcourse;
                }
            }

            return $ret;
        }


    /**
     * Find and return a list of course records.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param array $courseids An array of input course idnumber values.
     * @return array Return data (course record) to be converted into a specific
     *               data format for sending to the client.
     */
        function get_course($client, $sesskey, $courseids, $idfield = 'idnumber') {
            if (!$this->validate_client($client, $sesskey)) {
                return $this->error('Invalid client connection.');
            }

            $uid = $this->get_session_user($client);                
            $ret = array();

            foreach ($courseids as $courseid) {
                $rcourse = new stdClass;

            /// This database operation MIGHT throw an HTML error message,
            /// so we've got to catch that and send it back in an error
            /// request.
                ob_start();

                $course = get_record('course', $idfield, $courseid);

                if (ob_get_length() && trim(ob_get_contents())) {
                /// Return an error with  the contents of the output buffer.
                    $msg = trim(ob_get_clean());

                    $rcourse->error = 'Database error: ' . $msg;
                }
                ob_end_clean();

                if (!empty($course)) {
                    if (!$this->isteacher($course->id, $uid)) {
                        $rcourse->error = 'You do not have proper access to perform ' .
                                          'this operation.';
                    } else {
                        $rcourse = $course;
                    }
                } else {
                    $rcourse->error = 'Invalid course ID (' . $courseid . ')';
                }

            /// Return a completed request to the client.
                $ret[] = $rcourse;
            }

            return $ret;
        }


    /**
     * Find and return a list of student grade values.
     * 
     * @uses $CFG
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param string $userid The user ID number of the student.
     * @param array $courseids An array of input course idnumber values.
     * @return array Return data (student grade information) to be converted
     *               into a specific data format for sending to the client.
     */
        function get_grades($client, $sesskey, $userid, $courseids, $idfield = 'idnumber') {
            global $CFG;

            if (!$this->validate_client($client, $sesskey)) {
                return $this->error('Invalid client connection.');
            }

            $uid = $this->get_session_user($client);

            if (!$user = get_record('user', 'idnumber', $userid)) {
                return $this->error('Could not find user record (' . $userid. ').');
            }

            require_once($CFG->dirroot . '/grade/lib.php');

            $return = array();

        /// Find grade data for the requested IDs.
            foreach ($courseids as $cid) {
                $rgrade = new stdClass;
                $rgrade->error    = '';
                $rgrade->courseid = $cid;

            /// Get the student grades for each course requested.
                if ($course = get_record('course', $idfield, $cid)) {
                /// The grade functions require the course record object to be global.
                    $GLOBALS['course'] = $course;

                    $success     = false;
                    $group       = get_current_group($course->id);
                    $preferences = grade_get_preferences($course->id);
                
                    list($grades_by_student, $all_categories) = grade_get_formatted_grades();

                /// Process the returned grades and add this student's grade information to
                /// the return array.                
                    if (!empty($grades_by_student)) {
                        foreach ($grades_by_student as $grade) {
                            if ($grade['student_data']['idnumber'] == $userid) {
                                if (is_array($grade['uncategorised'])) {
                                    $rgrade->grades = $grade['uncategorised'];
                                    $success        = true;
                                }
                            }
                        }
                    }
                    
                    if (!$success) {
                        $rgrade->error = 'No grade data for student ' . fullname($user) .
                                         ' in course ' . $course->fullname;
                    }
                    

                } else {
                    $rgrade->error = 'Could not find course ' . $cid;
                }

                $return[] = $rgrade;
            }

            return $return;
        }


    /**
     * Enrol users as a student in the given course.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param string $courseid The course ID number to enrol students in.
     * @param array $userids An array of input user idnumber values for enrolment.
     * @return array Return data (user_student records) to be converted into a
     *               specific data format for sending to the client.
     */
        function enrol_students($client, $sesskey, $courseid, $userids, $idfield = 'idnumber') {
			if (DEBUG) $this->debug_output('enrol_students');
            if (!$this->validate_client($client, $sesskey)) {
                return $this->error('Invalid client connection.');
            }

            $uid = $this->get_session_user($client);

            if (!$course = get_record('course', 'idnumber', $courseid)) {
                return $this->error('Could not find course record (' . $courseid . ')');
            }

            $error  = '';
            $return = new stdClass;

            if ($course->enrolperiod) {
                $timestart = time();
                $timeend   = $timestart + $course->enrolperiod;
            } else {
                $timestart = $timeend = 0;
            }

            if (!empty($userids)) {
                foreach ($userids as $userid) {
                    if (!$user = get_record('user', $idfield, $userid)) {
                        $error .= 'Could not find user record (' . $userid. ').' . "\n";
                    } else {
                        if (!enrol_student($user->id, $course->id, $timestart, $timeend)) {
                            $error .= 'Could not enrol user ' . fullname($user) .
                                      ' in course ' . $course->longname . ".\n";
                        }
                    }
                }
            }

            $return->error    = $error;
            //$return->students = get_records('user_students', 'course', $course->id);

            return $return;
        }


      /**
     * Enrol users with role in the given course.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param string $courseid The course ID number to enrol students in.
     * @param array $userids An array of input user idnumber values for enrolment.
     * @param string $roleid The role (student, teacher, editingteacher)
     * @return array Return data (user_student records) to be converted into a
     *               specific data format for sending to the client.
     */
        function enrol($client, $sesskey, $courseid, $userids , $roleid ) {
			if (DEBUG) $this->debug_output('enrol');
            //error_log(__METHOD__, 1, "schewe@oncampus.de");
            if (DEBUG) $this->debug_output(__METHOD__.' Try to assign role (START) '.$client.', '.$sesskey.', '.$courseid.', '.$roleid);
            
            $error  = '';
            $return = new stdClass;
            
            if (!$this->validate_client($client, $sesskey)) {
                $return->error = 'Invalid client connection.';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }

            $uid = $this->get_session_user($client);

            if (!$course = get_record('course', 'idnumber', $courseid)) {
                $return->error = 'Could not find course record (' . $courseid . ')';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
    
            if ((0 == $course->category) || 'site' == $course->format) {
        		    $context = get_context_instance_by_id($course->id);
    		    }
            else if (!$context = get_context_instance(CONTEXT_COURSE, $course->id)) {
                $return->error = 'Could not find context for course record (' . $course->id . ')';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
    
            // student, teacher, editingteacher, coursecreator
            if (!$roles = get_roles_with_capability('moodle/legacy:'.$roleid, CAP_ALLOW)) {
                $return->error = 'Could not find role record (' . $roleid . ')';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            $role = array_shift($roles);
            
          $groupid = 0;
        $hidden = 0;
      $enrol = 'manual';
              
    $timestart = $timeend = 0;
    
    $result = 0;
    if (!empty($userids)) {
        foreach ($userids as $userid) {
            if (!$user = get_record('user', 'idnumber', $userid)) {
                $error .= 'Could not find user record: ' . $userid . "\n";
            } else {
                if (DEBUG) {
                  $this->debug_output(__METHOD__.' search:'.$userid.' found:'.var_export($user, true));
                }
                if (empty($userid) || $userid != $user->idnumber)
        			  {
        			     $error .= 'invalid user';
        			  } else {
                  if (!$result = role_assign($role->id, $user->id, $groupid, $context->id, $timestart, $timeend, $hidden, $enrol)) {
                      $error .= 'Could not enrol user ' . fullname($user) . ' in course ' . $course->longname . ".\n";
                  }
                  //AC-Assign
                  if (!aiw_Assign($userid, $courseid, $role->id)) {
                    	$error .= 'AC-Error Assign: '. $userid . $courseid . $roleid;
                  }
				  
				  // Profilfeld fr Kundenrolle ermitteln
				  $fieldid = get_field_select('user_info_field', 'id', "shortname = 'customerrole'");
				  // Kundenrolle fr User ermitteln
				  $whereclause="userid = '".$user->id."' and fieldid='".$fieldid."'";
				  $usercustomerrole = get_field_select('user_info_data', 'data', $whereclause);
				  
				  // falls es sich um einen Benutzer mit der Kundenrolle 'Business' handelt
				  // werden die Lernerfolge fr alle Tests und Scorm-Objekte des Users fr diesen Kurs gelscht
				  if ($usercustomerrole=='Business'){
				  	if (DEBUG) 
						{
						$this->debug_output('b2b_user');
						$this->debug_output('user_id:'.$user->id);
						$this->debug_output('course_id:'.$course->id);
						}
					//Zeitraum nach dem Lernerfolge gelscht werden
					$deltime=time(); // -23328000  akuelles Datum -270 Tage
					
					// alle Tests im Kurs ermitteln
					$quizes=get_records_sql('select id from mdl_quiz where course='.$course->id);
					foreach ($quizes as $quiz) 
						{
						// Lernerfolge fr Test lschen
						if (DEBUG) {$this->debug_output('quiz_id:'.$quiz->id);}
						$whereclause = " quiz = '".$quiz->id."' AND userid = '".$user->id."' AND timemodified < ".$deltime;
						if (DEBUG) {$this->debug_output('quizdel:'.$whereclause);}
				        if (delete_records_select('quiz_grades', $whereclause))
							{
							// nach erfolgreichem Lschen Eintrag in Log-Tabelle
							add_to_log($course->id, "quiz_grades", "delete", "", $quiz->id, "", $user->id);
							}
						}
					
					// alle Scorm-Objekte im Kurs ermitteln
					$scorms=get_records_sql('select id from mdl_scorm where course='.$course->id);
					foreach ($scorms as $scorm) 
						{
						// Lernerfolge fr Scorm-Objekt lschen
						if (DEBUG) {$this->debug_output('scorm_id:'.$scorm->id);}
				        $whereclause = " scormid = '".$scorm->id."' AND userid = '".$user->id."' AND timemodified < ".$deltime;
						if (DEBUG) {$this->debug_output('scormdel:'.$whereclause);}
				        if (delete_records_select('scorm_scoes_track', $whereclause))
							{
							// nach erfolgreichem Lschen Eintrag in Log-Tabelle
							add_to_log($course->id, "scorm_scoes_track", "delete", "", $scorm->id, "", $user->id);
							}
						}
				  }
               }
            }
        }
    }
    if (DEBUG) {
		  $assigned_role = get_record('role_assignments', 'userid', $user->id);
		  $this->debug_output(__METHOD__.' resp:'.$result." uid:".$user->id." ctx:".$context->id." ".var_export($assigned_role, true));
    }
	  $return->error    = $error;

	  if (DEBUG) {
		  $this->debug_output(__METHOD__.' err: '.$return->error);
	  }

    return $return;
  } //closes function enrol
    
     /**
     * Unenrol users from the given course.
     * 
     * @param int $client The client session ID.
     * @param string $sesskey The client session key.
     * @param string $courseid The course ID number to enrol students in.
     * @param array $userids An array of input user idnumber values for enrolment.
     * @return array Return data (user_student records) to be converted into a
     *               specific data format for sending to the client.
     */
        function unenrol($client, $sesskey, $courseid, $userids) {
            //error_log(__METHOD__, 1, "schewe@oncampus.de");
//          if (DEBUG) $this->debug_output(__METHOD__.' Try to unassign role (START) '.$client.', '.$sesskey.', '.$courseid.', '.$userids);
            
            $error  = '';
            $return = new stdClass;
            
            if (!$this->validate_client($client, $sesskey)) {
                $return->error = 'Invalid client connection.';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }

            $uid = $this->get_session_user($client);

//$this->debug_output(__METHOD__.' Try to unassign role (USER) '.$client.', '.$sesskey.', '.$courseid.', '.$userids);
            
            if (!$course = get_record('course', 'idnumber', $courseid)) {
                $return->error = 'Could not find course record (' . $courseid . ')';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }

//$this->debug_output(__METHOD__.' Try to unassign role (COURSE) '.$client.', '.$sesskey.', '.$courseid.', '.$userids);
            
            if (!$context = get_context_instance(CONTEXT_COURSE, $course->id)) {
                $return->error = 'Could not find context for course record (' . $course->id . ')';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
 
 //$this->debug_output(__METHOD__.' Try to unassign role (CONTEXT) '.$client.', '.$sesskey.', '.$courseid.', '.$userids);
                         
            $roleid = 0;
            $groupid = 0;
               
            $result = '';
            if (!empty($userids)) 
		        {
             		foreach ($userids as $array) //< $userid
			          {
			              $userid = $array['item'];
			               
                		if (DEBUG) $this->debug_output(__METHOD__.' Try to unassign role (user) '.$userid);
                		
		                if (!$user = get_record('user', 'idnumber', $userid)) 
		                {
                    		$error .= 'Could not find user record (' . $userid. ').' . "\n";
                    		if (DEBUG) $this->debug_output(__METHOD__.' Try to unassign role (error) '.$error);

                		} 
		                else 
		                {
                    		if (DEBUG) $this->debug_output(__METHOD__.' Try to unassign role (user) '.fullname($user));
                    		if (!$result = role_unassign($roleid, $user->id, $groupid, $context->id)) 
			                  {
                        		$error .= 'Could not unenrol user '.$userid.' from course ' . $course->longname . ".\n";
                       			if (DEBUG) $this->debug_output(__METHOD__.' Try to unassign role (error) '.$error);
                 		    }
              					//AC-Unassign
              					if (!aiw_UnAssign($userid, $courseid)) {
              						$error .= 'AC-Error UnAssign: '. $userid . $courseid;
              					}
             		    }
             		}
           	}
            
            $return->error    = $error;
            //$return->users = get_records('user', 'course', $course->id);
            
            return $return;
        }
        

////////////


        function import_backup($client, $sesskey, $courseid, $file)
        {
        	global $CFG;
			
			header("Content-Type: text/xml\r\n"); // added for moodle 1.9
        	
        	$return = new stdClass;
        	
        	if (!$this->validate_client($client, $sesskey)) {
                $return->error = 'Invalid client connection.';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            if (!$course = get_record('course', 'idnumber', $courseid)) {
                $return->error = 'Could not find course record (' . $courseid . ')';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            $path = '/home/moodle/moodle_mods/course_default/';
            $backup_file = $path.$file;
            
            if (!file_exists($backup_file))
            {
            	if (DEBUG) $this->debug_output('Backup file ('.$backup_file.') does not exist! Using default now!');
        		$lang_array = explode('_', $course->lang);
				$lang = $lang_array[0];
				switch ($lang)
				{
				    case 'de':
					$backup_file = $path.'course-default-de.zip';
					break;
				    
				    default:
				    case 'en':
					$backup_file = $path.'course-default-en.zip';
					break;
				}
            }
            
        	ob_start();
            include_once("$CFG->dirroot/lib/xmlize.php");
            include_once("$CFG->dirroot/backup/restorelib.php");
            include_once("$CFG->dirroot/backup/lib.php");
            
            //import_backup_file_silently($pathtofile,$destinationcourse,$emptyfirst=false,$userdata=false)
            $ok = import_backup_file_silently($backup_file, $course->id, true, false);
            
            if (DEBUG) $this->debug_output('Import backup for course ID: ' . $course->id.' ('.$backup_file.') = '.$ok);
            
            if (!$ok && ob_get_length() && trim(ob_get_contents())) {
            /// Return an error with  the contents of the output buffer.
                $msg            = trim(ob_get_contents());
                $return->error = 'Import error: ' . $msg;
                if (DEBUG) $this->debug_output($return->error);
            }
            ob_end_clean();
            
            return $return;
        }
        
        function add_to_metacourse($client, $sesskey, $courseid1, $courseid2) 
        {
            global $CFG;
            
            $error  = '';
            $return = new stdClass;
            
            if (!$this->validate_client($client, $sesskey)) {
                $return->error = 'Invalid client connection.';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            if (!$course1 = get_record('course', 'idnumber', $courseid1)) {
                $return->error = 'Could not find course record (' . $courseid1 . ')';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            if (!$course2 = get_record('course', 'idnumber', $courseid2)) {
                $return->error = 'Could not find course record (' . $courseid2 . ')';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            if (!add_to_metacourse($course2->id, $course1->id)) //< metacourse, course
            {
              $error .= 'Could not add ('.$course1->longname.') to meta course ('.$course2->idnumber.')';

              if (DEBUG) $this->debug_output(__METHOD__.$error);
            }
            
            // TODO: insert mdl_resource (link to meta course)
            $url = $CFG->wwwroot.'/course/view.php?id='.$course2->id;
            $this->add_resource_link($client, $sesskey, $courseid1, $url, $course2->fullname, $course2->summary, 'file');
            
            $return->error = $error;
            
            return $return;
        }
        
        function add_resource_link($client, $sesskey, $courseid, $url, $name, $summary = '', $type = 'file')
        {
        	global $CFG;
        	
            $return = new stdClass;
            
            if (!$this->validate_client($client, $sesskey)) {
                $return->error = 'Invalid client connection.';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            if (!$course = get_record('course', 'idnumber', $courseid)) {
                $return->error = 'Could not find course record (' . $courseid . ')';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            // insert mdl_course_modules
            
            $cm = new stdClass;
            $cm->course   = $course->id;
            $cm->module   = 13; //< mdl_resource
            //$cm->instance = ; // The instance ID
            //$cm->section  = ; // The section ID
            $cm->added    = time();
            
            if (!$module = get_record("modules", "id", $cm->module)) {
                $return->error = "This module doesn't exist";
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            if ('resource' != $module->name) {
                $return->error = "This module is not a 'resource'";
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            if ('repository' == $type)
            {
            	$url = "{$CFG->wwwroot}/mod/resource/type/repository/hive/hiveresources.php?aliasid={$url}&viewtype=original";
            }
            
            // insert mdl_resource
            $resource = new stdClass;
            $resource->course = $course->id;
            $resource->name = $name;
            $resource->type = $type;
            $resource->reference = $url;
            $resource->summary = $summary;
            $resource->timemodified = time();
        	if ('file' == $type)
	    	{
				$resource->alltext = 'useridnumber=uid,courseidnumber=cid';
	    	}

            if (!$cm->instance = insert_record("resource", $resource))
            {
              $return->error = 'Could not add ('.$url.') as resource for ('.$course->idnumber.')';
              if (DEBUG) $this->debug_output($return->error);
              return $return;
            }
            //$this->debug_output(__METHOD__.var_export($resource, true));
            /*if (!get_record("course_modules", "course", $cm->course, "module", $cm->module)) {
                if (DEBUG) $this->debug_output("This course module doesn't exist");
            }*/
            if (!$cm->id = add_course_module($cm))
            {
              $return->error = 'Could not add a new course module for ('.$course->idnumber.')';
              if (DEBUG) $this->debug_output($return->error);
              return $return;
            }
            //$this->debug_output(__METHOD__.var_export($cm, true));
            $mod = new stdClass;
            $mod->course = $course->id;
            //$mod->sequence = $cm->id;
            $mod->coursemodule = $cm->id;
            $mod->section = 1;//< 0 = below News Forum TODO: detect Forum in sequence
            
            if (!$cm->section = add_mod_to_section($mod))
            {
              $return->error = 'Could not add mod to section for ('.$course->idnumber.')';
              if (DEBUG) $this->debug_output($return->error);
              return $return;
            }
            //$this->debug_output(__METHOD__.var_export($mod, true));
            //$this->debug_output(__METHOD__.var_export($cm, true));
            if (!set_field("course_modules", "section", $cm->section, "id", $cm->id)) {
                $return->error = 'Could not update the course module with the correct section';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            rebuild_course_cache($course->id);
            
            return $return;
        }
         
        function update_resource_link($client, $sesskey, $courseid, $url, $name, $summary = '', $type = 'repository', $rname = 'Online Modul starten')
        {
        	global $CFG;
        	
            $return = new stdClass;
            
            if (!$this->validate_client($client, $sesskey)) {
                $return->error = 'Invalid client connection.';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            if (!$course = get_record('course', 'idnumber', $courseid)) {
                $return->error = 'Could not find course record (' . $courseid . ')';
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            if (!$resource = get_record("resource", "course", $course->id, "name", $rname)) {
                $return->error = "This resource doesn't exist ".$rname;
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            if ('repository' == $type)
            {
            	$url = "{$CFG->wwwroot}/mod/resource/type/repository/hive/hiveresources.php?aliasid={$url}&viewtype=original";
            }
            
            //$resource->name = $name;
            $resource->type = $type;
            $resource->reference = $url;//"{$CFG->wwwroot}/mod/resource/type/repository/hive/hiveresources.php?aliasid={$url}&viewtype=original";
            //$resource->summary = $summary;
            $resource->timemodified = time();
            
            if (!update_record("resource", $resource)) {
                $return->error = "Resource update failed";
                if (DEBUG) $this->debug_output($return->error);
                return $return;
            }
            
            rebuild_course_cache($course->id);
            
            return $return;
        }
        
    /**
     * Initializes a new server and calls the dispatch function upon an
     * incoming client request.
     * 
     * @todo Override in protocol-specific server subclass.
     * @param none
     * @return none
     */
        function main() {
        /// Override in protocol-specific server subclass.
        }


    /**
     * Sends an error response back to the client.
     * 
     * @todo Override in protocol-specific server subclass.
     * @param string $msg The error message to return.
     * @return An error message string.
     */
        function error($msg) {
            return $msg;  /// Override in protocol-specific subclass.
        }


    /**
     * Determines if a user an admin
     * 
     * - Copied from /lib/moodlelib.php & /lib/deprecatedlib.php without using $USER.
     * 
     * @param int $userid The id of the user as is found in the 'user' table
     * @return boolean
     */
        function isadmin($userid) {
            if ($this->using17) {
                return has_capability('moodle/legacy:admin',
                                      get_context_instance(CONTEXT_SYSTEM, SITEID), $userid, false);
            } else {
                return record_exists('user_admins', 'userid', $userid);
            }
        }


    /**
     * Determines if a user is a teacher (or better)
     * 
     * - Copied from /lib/moodlelib.php & /lib/deprecatedlib.php without using $USER.
     *
     * @param int $courseid The id of the course that is being viewed.
     * @param int $userid The id of the user that is being tested against.
     * @param boolean $includeadmin If true this function will return true when it encounters an admin user.
     * @return boolean
     */
        function isteacher($courseid, $userid, $includeadmin = true) {
            if ($includeadmin && $this->isadmin($userid)) {
                return true;
            }

            if ($this->using17) {
                return (has_capability('moodle/legacy:teacher',
                                       get_context_instance(CONTEXT_COURSE, $courseid), $userid, false) ||
                        has_capability('moodle/legacy:editingteacher',
                                       get_context_instance(CONTEXT_COURSE, $courseid), $userid, false));
            } else {
                return record_exists('user_teachers', 'userid', $userid, 'course', $courseid);
            }
        }


    /**
     * Determines if a user is a teacher in any course, or an admin
     *
     * - Copied from /lib/moodlelib.php & /lib/deprecatedlib.php without using $USER.
     * 
     * @param int $userid The id of the user that is being tested against.
     * @param boolean $includeadmin If true this function will return true when it encounters an admin user.
     * @return boolean
     */
        function isteacherinanycourse($userid, $includeadmin = true) {
            if ($includeadmin and $this->isadmin($userid)) {  // admins can do anything
                return true;
            }

            if (!record_exists('role_assignments', 'userid', $userid)) {    // Has no roles anywhere
                return false;
            }

        /// If this user is assigned as an editing teacher anywhere then return true
            if ($roles = get_roles_with_capability('moodle/legacy:editingteacher', CAP_ALLOW)) {
                foreach ($roles as $role) {
                    if (record_exists('role_assignments', 'roleid', $role->id, 'userid', $userid)) {
                        return true;
                    }
                }
            }

        /// If this user is assigned as a non-editing teacher anywhere then return true
            if ($roles = get_roles_with_capability('moodle/legacy:teacher', CAP_ALLOW)) {
                foreach ($roles as $role) {
                    if (record_exists('role_assignments', 'roleid', $role->id, 'userid', $userid)) {
                        return true;
                    }
                }
            }

            return record_exists('user_teachers', 'userid', $userid);
        }


    /**
     * Determines if a user is allowed to edit a given course
     *
     * - Copied from /lib/moodlelib.php & /lib/deprecatedlib.php without using $USER.
     * 
     * @param int $courseid The id of the course that is being edited
     * @param int $userid The id of the user that is being tested against.
     * @return boolean
     */
        function isteacheredit($courseid, $userid) {
            if ($this->isadmin($userid)) {
                return true;
            }

            if ($this->using17) {
                return has_capability('moodle/legacy:editingteacher',
                                      get_context_instance(CONTEXT_COURSE, $courseid), $userid, false);
            } else {
                return get_field('user_teachers', 'editall', 'userid', $userid, 'course', $courseid);
            }
        }


    /**
     * Determines if a user can create new courses
     *
     * - Copied from /lib/moodlelib.php & /lib/deprecatedlib.php without using $USER.
     *
     * @param int $userid The user being tested.
     * @return boolean
     */
        function iscreator($userid) {
            if ($this->isadmin($userid)) {
                return true;
            }

            if ($this->using17) {
                return has_capability('moodle/legacy:coursecreator',
                                      get_context_instance(CONTEXT_SYSTEM, SITEID), $userid, false);
            } else {
                return record_exists('user_coursecreators', 'userid', $userid);
            }
        }


    /**
     * Do server-side debugging output (to file).
     * 
     * @uses $CFG
     * @param mixed $output Debugging output.
     * @return none
     */
        function debug_output($output) {
            global $CFG;
            
            $fp = fopen($CFG->dataroot . '/debug.out', 'a');
            fwrite($fp, "[" . time() . "] $output\n");
            fflush($fp);
            fclose($fp);
        }

    }

?>